SystemOrganization addCategory: #'GT-Spotter-EventRecorder'!
SystemOrganization addCategory: 'GT-Spotter-EventRecorder-Announcements'!
SystemOrganization addCategory: 'GT-Spotter-EventRecorder-Base'!
SystemOrganization addCategory: 'GT-Spotter-EventRecorder-Events'!
SystemOrganization addCategory: 'GT-Spotter-EventRecorder-Settings'!
SystemOrganization addCategory: 'GT-Spotter-EventRecorder-Tests'!

TestCase subclass: #GTSpotterEventCollectorBackwardCompatibilityTest
	instanceVariableNames: 'unpacking checkedEvents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Tests'!

!GTSpotterEventCollectorBackwardCompatibilityTest class methodsFor: 'serializing' stamp: 'JurajKubelka 10/30/2015 09:39'!
instancesForAllEvents
	"It instantiates all Spotter events and set all instance variables."
	^ GTSpotterRecorderEvent allSubclasses collect: [ :eachEventClass | | instance |
		instance := eachEventClass new.
		1 to: eachEventClass instSize do: [ :index |
			(instance instVarAt: index) ifNil: [ | variableName |
				variableName := eachEventClass instVarNameForIndex: index.
				(instance respondsTo: variableName) ifTrue: [ instance perform: variableName ] ].
			(instance instVarAt: index) ifNil: [ 
				instance instVarAt: index put: #value ] ].
		instance
	 ]! !

!GTSpotterEventCollectorBackwardCompatibilityTest class methodsFor: 'serializing' stamp: 'JurajKubelka 10/30/2015 09:34'!
serializeAnnouncementWithAllEvents
	"self serializeAnnouncementWithAllEvents"
	| collector |
	collector := GTSpotterEventCollector new.
	self instancesForAllEvents do: [ :eachEvent | collector add: eachEvent ].
	^ GTEventPacking default pack: collector.
	! !

!GTSpotterEventCollectorBackwardCompatibilityTest class methodsFor: 'serializing' stamp: 'JurajKubelka 10/30/2015 09:39'!
storeSerializedAnnouncementIntoMethod: aSymbol
	"self storeSerializedAnnouncementIntoMethod: #version3"
	| methodSource |
	methodSource := (String streamContents: [ :aStream |
		aStream 
			nextPutAll: aSymbol asString; cr; 
			tab; 
			nextPutAll: '"Automatically created by class method #storeSerializedAnnouncementIntoMethod: ';
			print: aSymbol;
			nextPut: $.;
			cr; tab;
			nextPutAll: 'It is a serialized instance of GTEventAnnouncement.';
			cr; tab;
			nextPutAll: 'It contains all instances of the Spotter recorder events."';
			cr; tab;
			nextPutAll: '<spotterEventRecorderSerializedData>';
			cr; tab;
			nextPutAll: '^ '.
		self serializeAnnouncementWithAllEvents storeOn: aStream. ]).
	self compile: methodSource classified: 'serialized data'.! !

!GTSpotterEventCollectorBackwardCompatibilityTest methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 09:44'!
allPragmaVersions
	^ Pragma allNamed: #spotterEventRecorderSerializedData in: self class.! !

!GTSpotterEventCollectorBackwardCompatibilityTest methodsFor: 'asserting' stamp: 'JurajKubelka 10/30/2015 12:32'!
assertAnnouncement: announcement
	self assert: announcement notNil.
	self assert: (announcement isKindOf: GTSpotterRecorderDataAnnouncement).
	announcement materializedData do: [ :eachEvent |
		"Check that all instance variables have some value."
		1 to: eachEvent class instSize do: [ :index |
			self 
				assert: (eachEvent instVarAt: index) notNil 
				description: [ String streamContents: [ :aStream |
						aStream 
							nextPutAll: 'Instance variable called ';
							print: (eachEvent class instVarNameForIndex: index);
							nextPutAll: ' of instance ';
							nextPutAll: eachEvent className;
							nextPutAll: ' is nil.' ] ]
		]
	]! !

!GTSpotterEventCollectorBackwardCompatibilityTest methodsFor: 'asserting' stamp: 'JurajKubelka 10/30/2015 12:32'!
assertVersion: aMethodName
	| announcement |
	announcement := unpacking unpack: (self perform: aMethodName).
	self assertAnnouncement: announcement.
	checkedEvents addAll: (announcement materializedData collect: #class).! !

!GTSpotterEventCollectorBackwardCompatibilityTest methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 12:30'!
materializePragmaVersion: aPragma
	^ unpacking unpack: (self perform: aPragma selector).! !

!GTSpotterEventCollectorBackwardCompatibilityTest methodsFor: 'running' stamp: 'JurajKubelka 10/30/2015 12:29'!
setUp
	super setUp.
	unpacking := GTEventUnpacking default.! !

!GTSpotterEventCollectorBackwardCompatibilityTest methodsFor: 'tests' stamp: 'JurajKubelka 10/30/2015 09:45'!
testDateAndTime
	self versionsDo: [ :announcement :version |
		announcement materializedData do: [ :eachEvent | | dateAndTime |
			dateAndTime := eachEvent dateAndTime.
			self assert: dateAndTime class equals: DateAndTime ] ]! !

!GTSpotterEventCollectorBackwardCompatibilityTest methodsFor: 'tests' stamp: 'JurajKubelka 10/30/2015 12:31'!
testMaterializeVersions
	| versions notMaterializedEvents |
	checkedEvents := Set new.
	versions := self allPragmaVersions.
	self assert: versions notEmpty.
	versions do: [ :eachPragma |
		self assertVersion: eachPragma selector ].
	self assert: checkedEvents notEmpty.
	notMaterializedEvents := GTSpotterRecorderEvent allSubclasses 
		removeAll: checkedEvents; 
		yourself.
	self 
		assert: notMaterializedEvents isEmpty
		description: [ 
			'Some events are not tested for backward compatibility.', String cr,
			'You should create new version using method #storeSerializedAnnouncementIntoMethod:.', String cr,
			'Do not override the existing serialized versions!!'
		]! !

!GTSpotterEventCollectorBackwardCompatibilityTest methodsFor: 'serialized data' stamp: 'JurajKubelka 10/30/2015 09:41'!
version1
	"Automatically created by class method #storeSerializedAnnouncementIntoMethod: #version1.
	It is a serialized event recorder announcement GTSpotterRecorderDataAnnouncement.
	It contains all instances of the recorder events."
	<spotterEventRecorderSerializedData>
	^ #[31 139 8 0 0 0 0 0 0 0 115 11 117 245 97 56 196 200 192 192 192 207 0 1 236 34 110 62 238 57 249 73 137 57 206 57 137 197 197 206 57 165 197 37 169 69 64 9 86 14 55 31 143 212 196 148 212 34 33 207 148 212 188 146 204 146 74 151 204 228 146 204 252 188 196 162 74 46 167 202 146 212 224 202 220 164 252 28 65 255 34 160 154 212 20 231 252 156 156 84 176 60 127 104 94 74 106 90 102 94 106 138 127 82 22 80 72 206 205 39 32 191 56 179 36 179 44 213 34 56 55 49 39 199 51 175 36 53 61 181 8 97 23 19 3 163 24 208 182 252 252 236 128 162 204 92 176 74 132 36 35 51 127 81 106 114 62 200 150 176 212 162 98 160 5 120 212 178 2 189 227 150 89 1 179 26 73 134 145 69 178 160 40 213 55 17 200 205 76 204 201 172 74 4 57 213 17 236 224 98 193 196 148 148 76 176 215 114 32 250 138 165 10 242 139 75 176 43 230 207 69 21 150 117 243 65 4 12 34 20 144 172 102 2 17 146 110 62 193 153 185 5 57 169 216 148 48 177 64 163 131 129 135 155 151 19 164 156 131 29 38 194 5 196 146 48 54 174 216 98 227 4 197 136 99 81 81 34 52 110 74 138 50 243 210 145 162 137 219 5 232 106 199 188 148 144 204 220 84 69 247 144 224 130 252 18 160 206 32 104 192 2 37 19 29 243 242 242 75 243 146 83 115 129 145 205 133 240 145 60 34 246 12 205 176 71 31 227 188 62 220 81 194 196 196 17 144 145 88 148 111 162 103 160 98 106 145 98 98 148 146 104 166 155 108 105 105 172 107 146 152 98 168 107 97 102 100 162 107 154 150 98 102 100 97 110 105 108 102 104 64 68 106 97 228 192 99 29 11 131 106 157 148 252 233 59 27 24 246 6 255 191 122 1 28 108 64 17 163 61 42 55 24 214 231 67 69 112 235 231 96 230 204 204 77 76 79 245 72 44 206 96 73 1 6 139 112 113 106 49 40 209 57 23 165 130 163 27 20 128 60 201 249 185 5 165 64 13 161 161 158 46 156 37 64 145 226 146 196 220 2 145 212 50 96 216 5 161 38 86 158 28 96 192 23 151 132 22 0 13 75 229 1 155 13 149 65 10 89 99 35 28 33 203 186 213 79 71 216 205 7 20 143 152 41 26 232 219 38 249 110 14 104 218 152 58 37 55 252 178 129 200 241 221 59 185 122 30 205 8 19 17 16 112 172 113 99 149 255 160 58 77 180 204 207 246 113 192 135 218 109 113 17 38 190 89 147 248 158 29 255 172 252 147 211 103 247 217 115 165 115 85 159 111 137 46 174 255 121 255 78 245 228 236 249 87 101 222 134 59 94 222 210 117 77 58 120 217 91 25 161 107 46 71 148 60 56 186 91 156 152 194 217 106 111 149 245 49 84 30 200 191 110 187 64 232 244 59 198 60 207 51 26 143 238 126 122 190 142 101 251 107 115 201 149 102 59 146 14 108 150 80 20 184 173 157 126 165 185 216 231 96 182 201 254 149 43 67 111 91 166 49 121 223 105 180 189 238 222 114 101 101 11 247 131 206 245 139 194 34 2 238 73 252 174 59 179 81 126 114 231 198 130 197 135 22 30 85 210 143 111 228 126 154 192 180 122 117 65 78 141 105 129 109 138 230 20 49 102 190 248 9 191 142 116 60 152 175 113 235 85 221 158 196 64 215 121 237 143 47 120 228 207 141 224 154 49 225 241 195 174 255 239 191 116 189 85 233 244 220 100 240 100 151 196 94 177 220 227 71 235 28 18 100 130 116 190 158 94 40 185 185 216 88 208 206 214 116 134 230 234 229 247 254 215 171 175 142 250 60 187 125 121 222 129 37 191 172 94 170 159 78 210 58 27 126 41 75 185 138 49 110 237 226 172 25 109 124 33 191 94 93 9 18 140 53 184 215 187 112 225 173 105 147 15 92 74 189 119 66 96 181 224 135 169 62 107 182 206 75 234 219 192 126 92 198 74 66 47 57 41 117 141 151 242 153 168 247 219 110 62 253 80 117 215 205 229 64 217 172 230 140 217 89 251 59 170 110 4 127 228 125 158 168 61 119 77 29 27 219 105 163 13 183 68 167 253 158 178 105 194 182 73 10 211 131 34 148 25 102 77 44 16 23 200 223 154 235 179 49 227 251 93 157 229 10 165 151 164 108 27 57 221 247 206 121 30 39 57 249 221 212 123 142 42 45 91 93 210 202 211 247 244 21 110 251 115 123 239 185 201 181 220 95 37 59 107 5 132 126 58 114 94 46 78 75 40 20 50 239 214 152 59 53 236 196 206 95 182 161 207 183 173 235 249 236 24 146 41 214 117 208 101 89 222 70 139 69 179 61 239 157 241 245 123 170 182 188 120 178 241 206 147 55 239 164 44 53 113 122 101 239 20 96 107 245 96 13 171 147 129 158 209 244 115 79 203 138 180 205 51 123 242 210 215 93 221 31 195 151 197 100 184 224 186 188 78 105 103 202 178 19 127 86 236 254 95 22 249 56 159 251 172 181 90 217 147 38 185 204 117 111 195 124 139 31 30 254 116 124 234 219 171 123 217 223 191 223 91 189 35 121 150 247 250 119 251 118 254 191 102 34 30 159 125 93 254 216 105 241 215 253 102 114 246 183 107 254 125 57 121 109 185 125 101 228 132 199 19 128 165 20 206 18 151 149 145 43 5 94 140 16 46 37 217 64 185 15 84 208 113 136 114 8 114 10 240 10 115 9 241 240 137 241 139 139 176 75 0 0 84 138 255 137 183 6 0 0]! !

!GTSpotterEventCollectorBackwardCompatibilityTest methodsFor: 'serialized data' stamp: 'JurajKubelka 10/30/2015 09:42'!
version2
	"Automatically created by class method #storeSerializedAnnouncementIntoMethod: #version2.
	It is a serialized event recorder announcement GTSpotterRecorderDataAnnouncement.
	It contains all instances of the recorder events."
	<spotterEventRecorderSerializedData>
	^ #[31 139 8 0 0 0 0 0 0 0 115 11 117 245 97 56 196 200 192 192 192 207 0 1 236 34 110 62 238 57 249 73 137 57 206 57 137 197 197 206 57 165 197 37 169 69 64 9 86 14 55 31 143 212 196 148 212 34 33 207 148 212 188 146 204 146 74 151 204 228 146 204 252 188 196 162 74 46 167 202 146 212 224 202 220 164 252 28 65 255 34 160 154 212 20 231 252 156 156 84 176 60 127 104 94 74 106 90 102 94 106 138 127 82 22 80 72 206 205 39 32 191 56 179 36 179 44 213 34 56 55 49 39 199 51 175 36 53 61 181 8 97 23 51 3 35 147 24 208 186 252 252 236 128 162 204 92 176 82 132 44 35 51 127 81 106 114 62 200 154 176 212 162 98 160 13 120 212 178 2 253 227 150 89 1 179 27 73 134 145 69 178 160 40 213 55 17 200 205 76 204 201 172 74 4 185 213 17 236 226 98 193 196 148 148 76 176 223 114 32 250 138 165 10 242 139 75 176 43 230 207 69 21 150 117 243 65 132 12 34 24 144 172 102 2 17 146 110 62 193 153 185 5 57 169 216 148 48 177 64 227 131 129 151 135 143 11 164 156 147 3 38 194 13 196 146 80 54 23 174 232 98 227 4 69 137 99 81 81 34 52 114 74 138 50 243 210 145 226 137 219 5 232 106 199 188 148 144 204 220 84 69 247 144 224 130 252 18 160 206 32 104 192 2 37 19 29 243 242 242 75 243 146 83 115 129 177 205 133 240 145 60 34 250 12 205 176 199 31 227 188 41 184 163 132 137 73 197 212 34 197 196 40 37 209 76 55 217 210 210 88 215 36 49 197 80 215 194 204 200 68 215 52 45 197 204 200 194 220 210 216 204 208 128 35 32 35 177 40 223 68 207 128 112 114 97 226 192 147 90 128 33 169 90 167 108 154 114 107 2 195 150 245 255 175 94 0 7 27 80 68 50 55 170 131 97 109 25 84 4 183 126 14 102 206 204 220 196 244 84 143 196 226 12 150 20 96 176 8 23 167 22 131 18 157 115 81 42 56 186 65 1 200 147 156 159 91 80 10 212 16 26 234 233 194 89 2 20 41 46 73 204 45 16 73 45 3 134 93 16 106 98 229 201 1 6 124 113 73 104 1 208 176 84 30 176 217 80 25 164 144 53 54 194 17 178 172 91 253 116 132 221 124 64 241 136 153 162 129 190 125 38 223 13 75 40 83 67 125 131 133 13 37 238 77 91 231 16 169 241 212 155 177 112 77 77 71 124 71 87 211 188 35 151 90 173 74 79 5 124 152 87 22 159 97 50 123 205 201 109 1 252 114 50 252 247 246 252 146 183 169 184 32 121 92 114 191 127 220 156 143 139 246 31 94 108 126 54 47 58 106 123 78 164 241 229 187 239 38 171 159 159 188 39 255 147 219 125 175 212 41 250 138 174 129 73 94 91 75 249 94 206 12 61 244 169 67 133 83 72 201 211 235 16 155 158 245 21 61 219 205 108 39 24 236 38 152 53 220 186 177 110 205 131 166 94 73 215 61 66 123 67 55 42 134 27 126 238 57 178 57 166 232 128 135 145 211 129 183 253 185 129 204 219 12 89 149 103 239 87 95 40 92 149 54 151 51 231 177 145 206 83 31 23 143 89 183 54 48 239 121 120 117 97 203 33 229 75 125 27 127 191 150 153 173 183 245 162 244 171 142 66 13 131 227 207 244 10 20 101 111 21 132 238 148 147 125 127 117 239 236 42 167 137 129 221 135 101 85 236 13 149 126 40 22 61 219 245 230 92 196 145 137 137 9 229 185 75 172 252 122 34 125 110 176 79 78 126 182 255 67 203 188 76 151 101 130 109 106 175 55 124 150 184 106 108 153 19 197 194 183 168 69 121 107 141 198 222 168 248 43 11 203 203 175 78 148 251 185 59 246 183 253 139 233 235 126 92 121 149 118 46 105 219 159 91 43 88 181 79 166 204 249 254 38 69 240 162 172 65 80 234 204 35 97 151 186 120 226 103 169 61 237 42 147 88 191 99 201 178 39 110 210 220 217 103 63 47 233 143 232 106 208 76 141 89 235 194 157 206 43 189 225 220 194 158 196 235 169 95 214 120 188 79 205 202 73 18 86 227 11 57 51 237 207 150 34 215 109 183 215 44 125 213 193 49 35 251 105 163 113 170 101 185 11 111 142 199 254 103 51 206 108 97 91 39 214 251 203 64 62 42 139 255 246 233 41 155 132 116 77 196 108 174 57 157 142 114 168 109 154 207 124 132 219 85 54 88 167 203 116 246 71 177 73 119 74 111 174 106 248 191 237 250 174 41 47 221 39 115 191 23 159 167 189 62 254 223 207 143 119 239 106 243 125 122 117 116 15 251 79 131 138 203 221 54 149 175 28 237 254 198 124 86 152 47 94 95 57 241 248 102 185 39 143 11 246 139 87 86 95 106 222 243 243 219 213 215 95 223 108 103 175 84 191 255 219 99 246 226 123 197 21 137 156 218 81 187 119 45 154 38 168 217 88 127 52 96 119 238 78 237 214 94 31 237 166 151 102 151 55 220 40 208 158 61 249 93 196 194 35 59 107 242 79 25 79 155 54 247 239 180 205 194 145 158 87 24 227 239 22 220 88 252 250 180 141 208 140 155 146 51 110 202 206 144 228 61 115 133 201 248 183 178 220 178 95 11 88 62 57 109 180 208 110 16 232 225 190 191 176 182 209 55 169 113 230 222 123 47 191 136 101 181 219 55 127 114 48 89 113 109 211 18 155 39 219 214 123 220 201 46 127 209 187 224 237 244 223 11 214 190 125 43 24 36 123 182 243 168 112 231 145 3 151 46 124 221 248 243 225 206 100 110 247 245 61 49 126 42 110 250 114 251 77 234 116 39 74 236 89 245 167 254 178 222 28 123 83 75 211 226 178 95 119 102 222 188 108 254 143 233 208 207 45 89 182 192 4 142 179 84 103 101 228 74 129 23 85 132 75 98 54 80 14 7 21 166 28 162 156 130 28 2 188 194 220 66 60 124 98 252 226 34 236 18 0 254 125 218 6 28 7 0 0]! !

!GTSpotterEventCollectorBackwardCompatibilityTest methodsFor: 'enumerating' stamp: 'JurajKubelka 10/30/2015 09:42'!
versionsDo: aBlock
	self allPragmaVersions do: [ :eachPragmaVersion | | announcement |
		announcement := self materializePragmaVersion: eachPragmaVersion.
		aBlock cull: announcement cull: eachPragmaVersion selector.
	]! !

TestCase subclass: #GTSpotterEventCollectorTest
	instanceVariableNames: 'collector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Tests'!

!GTSpotterEventCollectorTest commentStamp: '<historical>' prior: 0!
A GTSpotterEventCollectorTest is a test class for testing the behavior of GTSpotterEventCollector!

!GTSpotterEventCollectorTest methodsFor: 'running' stamp: 'JurajKubelka 10/11/2015 14:09'!
setUp
	super setUp.
	collector := GTSpotterEventCollector new! !

!GTSpotterEventCollectorTest methodsFor: 'tests' stamp: 'JurajKubelka 10/30/2015 12:36'!
testBundle
	| events bundle |
	"collecting data"
	events := OrderedCollection new.
	events 
		add: GTSpotterRecorderOpenEvent new;
		add: GTSpotterRecorderShowPreviewEvent new;
		add: GTSpotterRecorderExitAnnouncementEvent new;
		add: GTSpotterRecorderOpenEvent new;
		add: GTSpotterRecorderShowPreviewEvent new;
		add: GTSpotterRecorderExitAnnouncementEvent new;
		add: GTSpotterRecorderOpenEvent new;
		add: GTSpotterRecorderShowPreviewEvent new.
	events do: [ :each | collector add: each ].
	"first delivery"
	bundle := collector bundle.
	self assert: collector postponedEvents size equals: 2.
	7 to: 8 do: [ :index |
		self assert: (events at: index) == (collector postponedEvents at: index - 6) ].
	self assert: bundle size equals: 6.
	bundle withIndexDo: [ :eachEvent :index |
		self assert: (events at: index) class = eachEvent class ].
	"second delivery"
	bundle := collector bundle.
	self assert: collector postponedEvents size equals: 2.
	7 to: 8 do: [ :index |
		self assert: (events at: index) == (collector postponedEvents at: index - 6) ].
	self assert: bundle isEmpty.
	"third delivery"
	collector add: GTSpotterRecorderExitAnnouncementEvent new.
	bundle := collector bundle.
	self assert: collector postponedEvents isNil.
	self assert: bundle size equals: 3.
	self assert: bundle first class equals: (events at: 7) class.
	self assert: bundle second class equals: (events at: 8) class.
	self assert: bundle third class equals: GTSpotterRecorderExitAnnouncementEvent.! !

!GTSpotterEventCollectorTest methodsFor: 'tests' stamp: 'JurajKubelka 10/11/2015 13:34'!
testPostponedEvents
	self assert: collector postponedEvents isNil! !

TestCase subclass: #GTSpotterEventRecorderTest
	instanceVariableNames: 'recorder checkedEvents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Tests'!

!GTSpotterEventRecorderTest commentStamp: '<historical>' prior: 0!
A GTSpotterEventRecorderTest is a test class for testing the behavior of GTSpotterEventRecorder!

!GTSpotterEventRecorderTest class methodsFor: 'accessing' stamp: 'JurajKubelka 4/1/2015 09:52'!
instancesForAllEvents
	"We instantiate all events and set all instance variables."
	^ GTSpotterRecorderEvent allSubclasses collect: [ :eachEventClass | | instance |
		instance := eachEventClass new.
		1 to: eachEventClass instSize do: [ :index |
			(instance instVarAt: index) ifNil: [ | variableName |
				variableName := eachEventClass instVarNameForIndex: index.
				(instance respondsTo: variableName) ifTrue: [ instance perform: variableName ] ].
			(instance instVarAt: index) ifNil: [ 
				instance instVarAt: index put: #value ] ].
		instance
	 ]! !

!GTSpotterEventRecorderTest class methodsFor: 'accessing' stamp: 'JurajKubelka 4/1/2015 09:53'!
serializeAnnouncementWithAllEvents
	"self serializeAnnouncementWithAllEvents"
	| recorder |
	recorder := GTSpotterEventRecorder new.
	recorder stopDelivering. 
	^ recorder serializedAnnouncementFrom: self instancesForAllEvents.
	! !

!GTSpotterEventRecorderTest class methodsFor: 'accessing' stamp: 'JurajKubelka 4/10/2015 09:51'!
storeSerializedAnnouncementIntoMethod: aSymbol
	"self storeSerializedAnnouncementIntoMethod: #version2"
	| methodSource |
	methodSource := (String streamContents: [ :aStream |
		aStream 
			nextPutAll: aSymbol asString; cr; 
			tab; 
			nextPutAll: '"Automatically created by class method #storeSerializedAnnouncementIntoMethod: ';
			print: aSymbol;
			nextPut: $.;
			cr; tab;
			nextPutAll: 'It is a serialized event recorder announcement GTSpotterRecorderDataAnnouncement.';
			cr; tab;
			nextPutAll: 'It contatins all instances of the recorder events."';
			cr; tab;
			nextPutAll: '<spotterEventRecorderSerializedData>';
			cr; tab;
			nextPutAll: '^ '.
		self serializeAnnouncementWithAllEvents storeOn: aStream. ]).
	self compile: methodSource classified: 'serialized data'.! !

!GTSpotterEventRecorderTest methodsFor: 'accessing' stamp: 'JurajKubelka 4/10/2015 09:31'!
allPragmaVersions
	^ Pragma allNamed: #spotterEventRecorderSerializedData in: self class.! !

!GTSpotterEventRecorderTest methodsFor: 'asserting' stamp: 'JurajKubelka 4/1/2015 10:19'!
assertAnnouncement: announcement
	self assert: announcement notNil.
	self assert: (announcement isKindOf: GTSpotterRecorderDataAnnouncement).
	announcement materializedData do: [ :eachEvent |
		"Check that all instance variables have some value."
		1 to: eachEvent class instSize do: [ :index |
			self 
				assert: (eachEvent instVarAt: index) notNil 
				description: [ String streamContents: [ :aStream |
						aStream 
							nextPutAll: 'Instance variable called ';
							print: (eachEvent class instVarNameForIndex: index);
							nextPutAll: ' of instance ';
							nextPutAll: eachEvent className;
							nextPutAll: ' is nil.' ] ]
		]
	]! !

!GTSpotterEventRecorderTest methodsFor: 'asserting' stamp: 'JurajKubelka 4/1/2015 10:37'!
assertVersion: aMethodName
	| announcement |
	announcement := recorder materialize: (self perform: aMethodName).
	self assertAnnouncement: announcement.
	checkedEvents addAll: (announcement materializedData collect: #class).! !

!GTSpotterEventRecorderTest methodsFor: 'accessing' stamp: 'JurajKubelka 4/10/2015 09:27'!
materializePragmaVersion: aPragma
	^ recorder materialize: (self perform: aPragma selector).! !

!GTSpotterEventRecorderTest methodsFor: 'accessing' stamp: 'JurajKubelka 2/28/2015 15:32'!
recordedData
	^ GTSpotterRecorderEvent allSubclasses collect: #new! !

!GTSpotterEventRecorderTest methodsFor: 'running' stamp: 'JurajKubelka 2/28/2015 15:34'!
setUp
	super setUp.
	recorder := GTSpotterEventRecorder new.
	recorder stopDelivering.! !

!GTSpotterEventRecorderTest methodsFor: 'tests' stamp: 'JurajKubelka 4/10/2015 10:33'!
testDateAndTime
	self versionsDo: [ :announcement :version |
		announcement materializedData do: [ :eachEvent | | dateAndTime |
			dateAndTime := eachEvent dateAndTime.
			self assert: dateAndTime class equals: DateAndTime ] ]! !

!GTSpotterEventRecorderTest methodsFor: 'tests' stamp: 'JurajKubelka 4/10/2015 09:31'!
testMaterializeVersions
	| versions notMaterializedEvents |
	checkedEvents := Set new.
	versions := self allPragmaVersions.
	self assert: versions notEmpty.
	versions do: [ :eachPragma |
		self assertVersion: eachPragma selector ].
	self assert: checkedEvents notEmpty.
	notMaterializedEvents := GTSpotterRecorderEvent allSubclasses 
		removeAll: checkedEvents; 
		yourself.
	self 
		assert: notMaterializedEvents isEmpty
		description: [ 
			'Some events are not tested for backward compatibility.', String cr,
			'You should create new version using method #storeSerializedAnnouncementIntoMethod:.', String cr,
			'Do not override the existing serialized versions!!'
		]! !

!GTSpotterEventRecorderTest methodsFor: 'tests' stamp: 'JurajKubelka 3/29/2015 14:20'!
testPostponeSomeDataOnAct
	| maybeDeliveredData toBeDeliveredData postponedData |
	self assert: recorder postponedData isNil.
	maybeDeliveredData := recorder newCollectionWithPostponedData.
	self assert: maybeDeliveredData size equals: 0.
	maybeDeliveredData 
		add: GTSpotterRecorderOpenEvent new;
		add: GTSpotterRecorderShowPreviewEvent new;
		add: GTSpotterRecorderExitAnnouncementEvent new;
		add: GTSpotterRecorderOpenEvent new;
		add: GTSpotterRecorderShowPreviewEvent new;
		add: GTSpotterRecorderActOnEvent new;
		add: GTSpotterRecorderOpenEvent new;
		add: GTSpotterRecorderShowPreviewEvent new.
	toBeDeliveredData := recorder postponeSomeData: maybeDeliveredData.
	postponedData := recorder postponedData.
	self assert: toBeDeliveredData size equals: 6.
	self assert: postponedData size equals: 2.
	1 to: 6 do: [ :index |
		self assert: (maybeDeliveredData at: index) == (toBeDeliveredData at: index) ].
	7 to: 8 do: [ :index |
		self assert: (maybeDeliveredData at: index) == (postponedData at: index - 6) ].
	"second delivery"
	maybeDeliveredData := recorder newCollectionWithPostponedData.
	self assert: recorder postponedData isNil.
	self assert: maybeDeliveredData size equals: 2.
	1 to: 2 do: [ :index |
		self assert: (maybeDeliveredData at: index) == (postponedData at: index) ].
	self assert: recorder postponedData isNil.
	maybeDeliveredData
		add: GTSpotterRecorderActOnEvent new.
	toBeDeliveredData := recorder postponeSomeData: maybeDeliveredData.
	postponedData := recorder postponedData.
	self assert: toBeDeliveredData size equals: 3.
	self assert: postponedData isNil.
	1 to: 3 do: [ :index |
		self assert: (maybeDeliveredData at: index) == (toBeDeliveredData at: index) ].
! !

!GTSpotterEventRecorderTest methodsFor: 'tests' stamp: 'JurajKubelka 3/29/2015 14:19'!
testPostponeSomeDataOnExit
	| maybeDeliveredData toBeDeliveredData postponedData |
	self assert: recorder postponedData isNil.
	maybeDeliveredData := recorder newCollectionWithPostponedData.
	self assert: maybeDeliveredData size equals: 0.
	maybeDeliveredData 
		add: GTSpotterRecorderOpenEvent new;
		add: GTSpotterRecorderShowPreviewEvent new;
		add: GTSpotterRecorderExitAnnouncementEvent new;
		add: GTSpotterRecorderOpenEvent new;
		add: GTSpotterRecorderShowPreviewEvent new;
		add: GTSpotterRecorderExitAnnouncementEvent new;
		add: GTSpotterRecorderOpenEvent new;
		add: GTSpotterRecorderShowPreviewEvent new.
	toBeDeliveredData := recorder postponeSomeData: maybeDeliveredData.
	postponedData := recorder postponedData.
	self assert: toBeDeliveredData size equals: 6.
	self assert: postponedData size equals: 2.
	1 to: 6 do: [ :index |
		self assert: (maybeDeliveredData at: index) == (toBeDeliveredData at: index) ].
	7 to: 8 do: [ :index |
		self assert: (maybeDeliveredData at: index) == (postponedData at: index - 6) ].
	"second delivery"
	maybeDeliveredData := recorder newCollectionWithPostponedData.
	self assert: recorder postponedData isNil.
	self assert: maybeDeliveredData size equals: 2.
	1 to: 2 do: [ :index |
		self assert: (maybeDeliveredData at: index) == (postponedData at: index) ].
	self assert: recorder postponedData isNil.
	maybeDeliveredData
		add: GTSpotterRecorderExitAnnouncementEvent new.
	toBeDeliveredData := recorder postponeSomeData: maybeDeliveredData.
	postponedData := recorder postponedData.
	self assert: toBeDeliveredData size equals: 3.
	self assert: postponedData isNil.
	1 to: 3 do: [ :index |
		self assert: (maybeDeliveredData at: index) == (toBeDeliveredData at: index) ].
! !

!GTSpotterEventRecorderTest methodsFor: 'tests' stamp: 'JurajKubelka 3/29/2015 14:16'!
testPrimitivePrepareDeliveryData
	| maybeDeliveredData  postponedData bytes announcement events |
	self assert: recorder postponedData isNil.
	maybeDeliveredData := OrderedCollection new.
	maybeDeliveredData 
		add: GTSpotterRecorderOpenEvent new;
		add: GTSpotterRecorderShowPreviewEvent new;
		add: GTSpotterRecorderExitAnnouncementEvent new;
		add: GTSpotterRecorderOpenEvent new;
		add: GTSpotterRecorderShowPreviewEvent new;
		add: GTSpotterRecorderExitAnnouncementEvent new;
		add: GTSpotterRecorderOpenEvent new;
		add: GTSpotterRecorderShowPreviewEvent new.
	maybeDeliveredData do: [ :each | recorder recordedData nextPut: each ].
	recorder primitivePrepareDeliveryData.
	postponedData := recorder postponedData.
	self assert: postponedData size equals: 2.
	7 to: 8 do: [ :index |
		self assert: (maybeDeliveredData at: index) == (postponedData at: index - 6) ].
	"second delivery"
	recorder recordedData nextPut: GTSpotterRecorderExitAnnouncementEvent new.
	recorder primitivePrepareDeliveryData.
	postponedData := recorder postponedData.
	self assert: postponedData isNil.
	self assert: recorder deliveryQueue size equals: 2.
	"first delivered data"
	bytes := recorder deliveryQueue nextOrNil.
	self assert: bytes notNil.
	announcement := recorder materialize: bytes.
	events := announcement materializedData.
	self assert: events size equals: 6.
	events withIndexDo: [ :eachEvent :index |
		self assert: (maybeDeliveredData at: index) class = eachEvent class ].
	"second delivered data"
	bytes := recorder deliveryQueue nextOrNil.
	self assert: bytes notNil.
	announcement := recorder materialize: bytes.
	events := announcement materializedData.
	self assert: events size equals: 3.
	self assert: (maybeDeliveredData at: 7) class = events first class.
	self assert: (maybeDeliveredData at: 8) class = events second class.
	self assert: events third class = GTSpotterRecorderExitAnnouncementEvent.! !

!GTSpotterEventRecorderTest methodsFor: 'tests' stamp: 'JurajKubelka 2/28/2015 15:45'!
testSerializedAnnouncementFrom
	| byteArray announcement |
	byteArray := recorder serializedAnnouncementFrom: self recordedData.
	announcement := recorder materializedAnnouncementFrom: byteArray.
	self assert: announcement class equals: GTSpotterRecorderDataAnnouncement.
	self assert: announcement materializedData isCollection.
	self assert: announcement materializedData size equals: self recordedData size.! !

!GTSpotterEventRecorderTest methodsFor: 'serialized data' stamp: 'JurajKubelka 4/1/2015 10:27'!
version1
	"Automatically created by class method #storeSerializedAnnouncementIntoMethod: #version1.
	It is a serialized event recorder announcement GTSpotterRecorderDataAnnouncement.
	It contatins all instances of the recorder events."
	<spotterEventRecorderSerializedData>
	^ #[31 139 8 0 0 0 0 0 0 0 115 11 117 245 97 56 196 200 192 192 192 207 0 1 236 34 110 62 238 57 249 73 137 57 206 57 137 197 197 206 57 165 197 37 169 69 64 9 86 14 55 31 143 212 196 148 212 34 33 207 148 212 188 146 204 146 74 151 204 228 146 204 252 188 196 162 74 46 167 202 146 212 224 202 220 164 252 28 65 255 34 160 154 212 20 231 252 156 156 84 176 60 127 104 94 74 106 90 102 94 106 138 127 82 22 80 72 206 205 39 32 191 56 179 36 179 44 213 34 56 55 49 39 199 51 175 36 53 61 181 8 97 23 19 3 163 24 208 182 252 252 236 128 162 204 92 176 74 132 36 35 51 127 81 106 114 62 200 150 176 212 162 98 160 5 120 212 178 2 189 227 150 89 1 179 26 73 134 145 69 178 160 40 213 55 17 200 205 76 204 201 172 74 4 57 213 17 236 224 98 193 196 148 148 76 176 215 114 32 250 138 165 10 242 139 75 176 43 230 207 69 21 150 117 243 65 4 12 34 20 144 172 102 2 17 146 110 62 193 153 185 5 57 169 216 148 48 177 64 163 131 129 135 155 151 19 164 156 131 29 38 194 5 196 146 48 54 174 216 98 227 4 197 136 99 81 81 34 52 110 74 138 50 243 210 145 162 137 219 5 232 106 199 188 148 144 204 220 84 69 247 144 224 130 252 18 160 206 32 104 192 2 37 19 29 243 242 242 75 243 146 83 115 129 145 205 133 240 145 60 34 246 12 205 176 71 31 227 188 62 220 81 194 196 196 17 144 145 88 148 111 162 103 160 98 106 145 98 98 148 146 104 166 155 108 105 105 172 107 146 152 98 168 107 97 102 100 162 107 154 150 98 102 100 97 110 105 108 102 104 64 68 106 97 228 192 99 29 11 131 106 157 148 252 233 59 27 24 246 6 255 191 122 1 28 108 64 17 163 61 42 55 24 214 231 67 69 112 235 231 96 230 204 204 77 76 79 245 72 44 206 96 73 1 6 139 112 113 106 49 40 209 57 23 165 130 163 27 20 128 60 201 249 185 5 165 64 13 161 161 158 46 156 37 64 145 226 146 196 220 2 145 212 50 96 216 5 161 38 86 158 28 96 192 23 151 132 22 0 13 75 229 1 155 13 149 65 10 89 99 35 28 33 203 186 213 79 71 216 205 7 20 143 152 41 26 232 219 38 249 110 14 104 218 152 58 37 55 252 178 129 200 241 221 59 185 122 30 205 8 19 17 16 112 172 113 99 149 255 160 58 77 180 204 207 246 113 192 135 218 109 113 17 38 190 89 147 248 158 29 255 172 252 147 211 103 247 217 115 165 115 85 159 111 137 46 174 255 121 255 78 245 228 236 249 87 101 222 134 59 94 222 210 117 77 58 120 217 91 25 161 107 46 71 148 60 56 186 91 156 152 194 217 106 111 149 245 49 84 30 200 191 110 187 64 232 244 59 198 60 207 51 26 143 238 126 122 190 142 101 251 107 115 201 149 102 59 146 14 108 150 80 20 184 173 157 126 165 185 216 231 96 182 201 254 149 43 67 111 91 166 49 121 223 105 180 189 238 222 114 101 101 11 247 131 206 245 139 194 34 2 238 73 252 174 59 179 81 126 114 231 198 130 197 135 22 30 85 210 143 111 228 126 154 192 180 122 117 65 78 141 105 129 109 138 230 20 49 102 190 248 9 191 142 116 60 152 175 113 235 85 221 158 196 64 215 121 237 143 47 120 228 207 141 224 154 49 225 241 195 174 255 239 191 116 189 85 233 244 220 100 240 100 151 196 94 177 220 227 71 235 28 18 100 130 116 190 158 94 40 185 185 216 88 208 206 214 116 134 230 234 229 247 254 215 171 175 142 250 60 187 125 121 222 129 37 191 172 94 170 159 78 210 58 27 126 41 75 185 138 49 110 237 226 172 25 109 124 33 191 94 93 9 18 140 53 184 215 187 112 225 173 105 147 15 92 74 189 119 66 96 181 224 135 169 62 107 182 206 75 234 219 192 126 92 198 74 66 47 57 41 117 141 151 242 153 168 247 219 110 62 253 80 117 215 205 229 64 217 172 230 140 217 89 251 59 170 110 4 127 228 125 158 168 61 119 77 29 27 219 105 163 13 183 68 167 253 158 178 105 194 182 73 10 211 131 34 148 25 102 77 44 16 23 200 223 154 235 179 49 227 251 93 157 229 10 165 151 164 108 27 57 221 247 206 121 30 39 57 249 221 212 123 142 42 45 91 93 210 202 211 247 244 21 110 251 115 123 239 185 201 181 220 95 37 59 107 5 132 126 58 114 94 46 78 75 40 20 50 239 214 152 59 53 236 196 206 95 182 161 207 183 173 235 249 236 24 146 41 214 117 208 101 89 222 70 139 69 179 61 239 157 241 245 123 170 182 188 120 178 241 206 147 55 239 164 44 53 113 122 101 239 20 96 107 245 96 13 171 147 129 158 209 244 115 79 203 138 180 205 51 123 242 210 215 93 221 31 195 151 197 100 184 224 186 188 78 105 103 202 178 19 127 86 236 254 95 22 249 56 159 251 172 181 90 217 147 38 185 204 117 111 195 124 139 31 30 254 116 124 234 219 171 123 217 223 191 223 91 189 35 121 150 247 250 119 251 118 254 191 102 34 30 159 125 93 254 216 105 241 215 253 102 114 246 183 107 254 125 57 121 109 185 125 101 228 132 199 19 128 165 20 206 18 151 149 145 43 5 94 140 16 46 37 217 64 185 15 84 208 113 136 114 8 114 10 240 10 115 9 241 240 137 241 139 139 176 75 0 0 84 138 255 137 183 6 0 0]! !

!GTSpotterEventRecorderTest methodsFor: 'serialized data' stamp: 'JurajKubelka 4/10/2015 09:50'!
version2
	"Automatically created by class method #storeSerializedAnnouncementIntoMethod: #version2.
	It is a serialized event recorder announcement GTSpotterRecorderDataAnnouncement.
	It contatins all instances of the recorder events."
	<spotterEventRecorderSerializedData>
	^ #[31 139 8 0 0 0 0 0 0 0 115 11 117 245 97 56 196 200 192 192 192 207 0 1 236 34 110 62 238 57 249 73 137 57 206 57 137 197 197 206 57 165 197 37 169 69 64 9 86 14 55 31 143 212 196 148 212 34 33 207 148 212 188 146 204 146 74 151 204 228 146 204 252 188 196 162 74 46 167 202 146 212 224 202 220 164 252 28 65 255 34 160 154 212 20 231 252 156 156 84 176 60 127 104 94 74 106 90 102 94 106 138 127 82 22 80 72 206 205 39 32 191 56 179 36 179 44 213 34 56 55 49 39 199 51 175 36 53 61 181 8 97 23 51 3 35 147 24 208 186 252 252 236 128 162 204 92 176 82 132 44 35 51 127 81 106 114 62 200 154 176 212 162 98 160 13 120 212 178 2 253 227 150 89 1 179 27 73 134 145 69 178 160 40 213 55 17 200 205 76 204 201 172 74 4 185 213 17 236 226 98 193 196 148 148 76 176 223 114 32 250 138 165 10 242 139 75 176 43 230 207 69 21 150 117 243 65 132 12 34 24 144 172 102 2 17 146 110 62 193 153 185 5 57 169 216 148 48 177 64 227 131 129 151 135 143 11 164 156 147 3 38 194 13 196 146 80 54 23 174 232 98 227 4 69 137 99 81 81 34 52 114 74 138 50 243 210 145 226 137 219 5 232 106 199 188 148 144 204 220 84 69 247 144 224 130 252 18 160 206 32 104 192 2 37 19 29 243 242 242 75 243 146 83 115 129 177 205 133 240 145 60 34 250 12 205 176 199 31 227 188 41 184 163 132 137 73 197 212 34 197 196 40 37 209 76 55 217 210 210 88 215 36 49 197 80 215 194 204 200 68 215 52 45 197 204 200 194 220 210 216 204 208 128 35 32 35 177 40 223 68 207 128 112 114 97 226 192 147 90 128 33 169 90 167 108 154 114 107 2 195 150 245 255 175 94 0 7 27 80 68 50 55 170 131 97 109 25 84 4 183 126 14 102 206 204 220 196 244 84 143 196 226 12 150 20 96 176 8 23 167 22 131 18 157 115 81 42 56 186 65 1 200 147 156 159 91 80 10 212 16 26 234 233 194 89 2 20 41 46 73 204 45 16 73 45 3 134 93 16 106 98 229 201 1 6 124 113 73 104 1 208 176 84 30 176 217 80 25 164 144 53 54 194 17 178 172 91 253 116 132 221 124 64 241 136 153 162 129 190 125 38 223 13 75 40 83 67 125 131 133 13 37 238 77 91 231 16 169 241 212 155 177 112 77 77 71 124 71 87 211 188 35 151 90 173 74 79 5 124 152 87 22 159 97 50 123 205 201 109 1 252 114 50 252 247 246 252 146 183 169 184 32 121 92 114 191 127 220 156 143 139 246 31 94 108 126 54 47 58 106 123 78 164 241 229 187 239 38 171 159 159 188 39 255 147 219 125 175 212 41 250 138 174 129 73 94 91 75 249 94 206 12 61 244 169 67 133 83 72 201 211 235 16 155 158 245 21 61 219 205 108 39 24 236 38 152 53 220 186 177 110 205 131 166 94 73 215 61 66 123 67 55 42 134 27 126 238 57 178 57 166 232 128 135 145 211 129 183 253 185 129 204 219 12 89 149 103 239 87 95 40 92 149 54 151 51 231 177 145 206 83 31 23 143 89 183 54 48 239 121 120 117 97 203 33 229 75 125 27 127 191 150 153 173 183 245 162 244 171 142 66 13 131 227 207 244 10 20 101 111 21 132 238 148 147 125 127 117 239 236 42 167 137 129 221 135 101 85 236 13 149 126 40 22 61 219 245 230 92 196 145 137 137 9 229 185 75 172 252 122 34 125 110 176 79 78 126 182 255 67 203 188 76 151 101 130 109 106 175 55 124 150 184 106 108 153 19 197 194 183 168 69 121 107 141 198 222 168 248 43 11 203 203 175 78 148 251 185 59 246 183 253 139 233 235 126 92 121 149 118 46 105 219 159 91 43 88 181 79 166 204 249 254 38 69 240 162 172 65 80 234 204 35 97 151 186 120 226 103 169 61 237 42 147 88 191 99 201 178 39 110 210 220 217 103 63 47 233 143 232 106 208 76 141 89 235 194 157 206 43 189 225 220 194 158 196 235 169 95 214 120 188 79 205 202 73 18 86 227 11 57 51 237 207 150 34 215 109 183 215 44 125 213 193 49 35 251 105 163 113 170 101 185 11 111 142 199 254 103 51 206 108 97 91 39 214 251 203 64 62 42 139 255 246 233 41 155 132 116 77 196 108 174 57 157 142 114 168 109 154 207 124 132 219 85 54 88 167 203 116 246 71 177 73 119 74 111 174 106 248 191 237 250 174 41 47 221 39 115 191 23 159 167 189 62 254 223 207 143 119 239 106 243 125 122 117 116 15 251 79 131 138 203 221 54 149 175 28 237 254 198 124 86 152 47 94 95 57 241 248 102 185 39 143 11 246 139 87 86 95 106 222 243 243 219 213 215 95 223 108 103 175 84 191 255 219 99 246 226 123 197 21 137 156 218 81 187 119 45 154 38 168 217 88 127 52 96 119 238 78 237 214 94 31 237 166 151 102 151 55 220 40 208 158 61 249 93 196 194 35 59 107 242 79 25 79 155 54 247 239 180 205 194 145 158 87 24 227 239 22 220 88 252 250 180 141 208 140 155 146 51 110 202 206 144 228 61 115 133 201 248 183 178 220 178 95 11 88 62 57 109 180 208 110 16 232 225 190 191 176 182 209 55 169 113 230 222 123 47 191 136 101 181 219 55 127 114 48 89 113 109 211 18 155 39 219 214 123 220 201 46 127 209 187 224 237 244 223 11 214 190 125 43 24 36 123 182 243 168 112 231 145 3 151 46 124 221 248 243 225 206 100 110 247 245 61 49 126 42 110 250 114 251 77 234 116 39 74 236 89 245 167 254 178 222 28 123 83 75 211 226 178 95 119 102 222 188 108 254 143 233 208 207 45 89 182 192 4 142 179 84 103 101 228 74 129 23 85 132 75 98 54 80 14 7 21 166 28 162 156 130 28 2 188 194 220 66 60 124 98 252 226 34 236 18 0 254 125 218 6 28 7 0 0]! !

!GTSpotterEventRecorderTest methodsFor: 'enumerating' stamp: 'JurajKubelka 4/10/2015 09:31'!
versionsDo: aBlock
	self allPragmaVersions do: [ :eachPragmaVersion | | announcement |
		announcement := self materializePragmaVersion: eachPragmaVersion.
		aBlock cull: announcement cull: eachPragmaVersion selector.
	]! !

!GTSpotterActOn methodsFor: '*gt-spotter-eventrecorder' stamp: 'YuriyTymchuk 8/30/2015 01:40'!
asRecorderEvent

	^ GTSpotterRecorderActOnEvent fromAnnouncement: self! !

!GTSpotterHidePreview methodsFor: '*gt-spotter-eventrecorder' stamp: 'YuriyTymchuk 8/30/2015 01:45'!
asRecorderEvent

	^ GTSpotterRecorderHidePreviewEvent fromAnnouncement: self! !

!GTSpotterCategorySelected methodsFor: '*gt-spotter-eventrecorder' stamp: 'YuriyTymchuk 8/30/2015 01:42'!
asRecorderEvent

	^ GTSpotterRecorderCategorySelectedEvent fromAnnouncement: self! !

GTEventCollector subclass: #GTSpotterEventCollector
	instanceVariableNames: 'postponedEvents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Base'!

!GTSpotterEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 14:02'!
bundle
	^ self postpone: (self bundle: (postponedEvents ifNil: [ OrderedCollection new ]))! !

!GTSpotterEventCollector methodsFor: 'initialization' stamp: 'JurajKubelka 10/11/2015 16:03'!
linkToSpotter: aSpotterModel
	aSpotterModel announcer weak when: Announcement send: #recordAnnouncement: to: self! !

!GTSpotterEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 13:57'!
postpone: aCollectionOfEvents
	| lastExitIndex |
	lastExitIndex := aCollectionOfEvents findLast: [ :each | each isCloseSpotter ].
	lastExitIndex = aCollectionOfEvents size ifTrue: [ 
		postponedEvents := nil.
		^ aCollectionOfEvents ].
	postponedEvents := aCollectionOfEvents copyFrom: lastExitIndex + 1 to: aCollectionOfEvents size.
	^ aCollectionOfEvents first: lastExitIndex.! !

!GTSpotterEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 13:34'!
postponedEvents
	"For testing/debugging purpose."
	^ postponedEvents! !

!GTSpotterEventCollector methodsFor: 'events' stamp: 'JurajKubelka 10/11/2015 16:04'!
recordAnnouncement: anAnnouncement
	Privacy sendDiagnosticsAndUsageData ifFalse: [ ^ self ].	
	(anAnnouncement respondsTo: #asRecorderEvent) ifTrue: [	
		self add: anAnnouncement asRecorderEvent ]
! !

!GTSpotterEventCollector methodsFor: 'events' stamp: 'JurajKubelka 10/11/2015 16:04'!
recordSpotterOpenEvent
	Privacy sendDiagnosticsAndUsageData ifFalse: [ ^ self ].	
	self add: GTSpotterRecorderOpenEvent new! !

!GTSpotterDiveInElement methodsFor: '*gt-spotter-eventrecorder' stamp: 'YuriyTymchuk 8/30/2015 01:44'!
asRecorderEvent

	^ GTSpotterRecorderDiveInElementEvent fromAnnouncement: self! !

!GTSpotterDiveInCategory methodsFor: '*gt-spotter-eventrecorder' stamp: 'YuriyTymchuk 8/30/2015 01:43'!
asRecorderEvent

	^ GTSpotterRecorderDiveInCategoryEvent fromAnnouncement: self! !

!GTSpotterDiveOut methodsFor: '*gt-spotter-eventrecorder' stamp: 'YuriyTymchuk 8/30/2015 01:44'!
asRecorderEvent

	^ GTSpotterRecorderDiveOutEvent fromAnnouncement: self! !

!GTSpotterContextChanged methodsFor: '*gt-spotter-eventrecorder' stamp: 'YuriyTymchuk 8/30/2015 01:43'!
asRecorderEvent

	^ GTSpotterRecorderContextChangedEvent fromAnnouncement: self! !

!GTSpotterSelectedCandidateChanged methodsFor: '*gt-spotter-eventrecorder' stamp: 'YuriyTymchuk 8/30/2015 01:46'!
asRecorderEvent

	^ GTSpotterRecorderSelectedCandidateChangedEvent fromAnnouncement: self! !

!GTSpotterShowPreview methodsFor: '*gt-spotter-eventrecorder' stamp: 'YuriyTymchuk 8/30/2015 01:46'!
asRecorderEvent

	^ GTSpotterRecorderShowPreviewEvent fromAnnouncement: self! !

!GTSpotterSearchFinished methodsFor: '*gt-spotter-eventrecorder' stamp: 'YuriyTymchuk 8/30/2015 01:47'!
asRecorderEvent

	^ GTSpotterRecorderSearchFinishedEvent fromAnnouncement: self! !

Announcement subclass: #GTSpotterRecorderDataAnnouncement
	instanceVariableNames: 'dictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Announcements'!

!GTSpotterRecorderDataAnnouncement commentStamp: 'AndreiChis 3/2/2015 15:31' prior: 0!
A GTSpotterRecorderDataAnnouncement is an announcement that  groups  all  usage data  that is send over the network.  Appart from the actual recorded data in includes some meta-data about the image and the UUID of the computer.

Instance Variables
	computerUUID:		<Object>
	data:		<Object>
	imageHash:		<Object>
	imageVersion:		<Object>
	latestUpdate:		<Object>
	sessionCreationTime:		<Object>
	timestamp:		<Object>

computerUUID
	- xxxxx

data
	- xxxxx

imageHash
	- xxxxx

imageVersion
	- xxxxx

latestUpdate
	- xxxxx

sessionCreationTime
	- xxxxx

timestamp
	- xxxxx
!

!GTSpotterRecorderDataAnnouncement class methodsFor: 'examples' stamp: 'MarcusDenker 7/1/2015 11:53'!
gtExampleAnnouncement
	<gtExample>
	^ self new
			data: #[ 1 2 3 4 ];
			timestamp: DateAndTime now;
			imageHash: SmalltalkImage current imagePath hash;
			sessionCreationTime: SmalltalkImage current session creationTime;
			imageVersion: SystemVersion current version;
			latestUpdate: SystemVersion current highestUpdate;
			computerUUID: GTSpotterEventRecorderSettings uniqueInstance computerUUID asString;
			eventRecorderVersion: GTSpotterEventRecorder version;
			yourself! !

!GTSpotterRecorderDataAnnouncement class methodsFor: 'examples' stamp: 'StefanReichhart 4/9/2015 12:01'!
gtExampleEmptyAnnouncement
	<gtExample>
	^ self new! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'updating' stamp: 'JurajKubelka 3/28/2015 17:49'!
addInfoFor: aGTSpotterRecorderEvent 
	dictionary keysAndValuesDo: [ :key :value |
		key = #data ifFalse: [ 
			aGTSpotterRecorderEvent infoAt: key put: value ] ]! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'converting' stamp: 'JurajKubelka 3/29/2015 12:19'!
associations
	^ dictionary 
			ifNil: [ #() ]
			ifNotNil: [ dictionary associations ]! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:25'!
at: key
	^ self dictionary at: key ifAbsent: [ nil ]! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:26'!
at: key put: value
	^ self dictionary at: key put: value! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:26'!
computerUUID
	^ self at: #computerUUID! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:26'!
computerUUID: anObject
	self at: #computerUUID put: anObject! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:27'!
data
	^ self at: #data! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:27'!
data: aByteArray 
	self at: #data put: aByteArray! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:25'!
dictionary
	^ dictionary ifNil: [ dictionary := Dictionary new ]! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:27'!
eventRecorderVersion
	^ self at: #eventRecorderVersion! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:28'!
eventRecorderVersion: anObject
	self at: #eventRecorderVersion put: anObject! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 17:45'!
events
	| events |
	events := self materializedData.
	events do: [ :each | self addInfoFor: each ].
	^ events! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'gt-inspector-extension' stamp: 'JurajKubelka 3/29/2015 12:18'!
gtInspectorItems
	^ OrderedCollection withAll: self associations! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'gt-inspector-extension' stamp: 'JurajKubelka 3/29/2015 12:19'!
gtInspectorItemsIn: composite
	<gtInspectorPresentationOrder: -1>  
	^ (composite table)
		title: 'Info';
		display: [ :anEvent | anEvent gtInspectorItems ];
		children: [ :each | 
			(each value isDictionary) 
				ifTrue: [ each value associations ] 
				ifFalse: [ (each value isArray and: [ each value allSatisfy: #isDictionary ])
						ifTrue: [ each value collectWithIndex: [:x :i | i -> x ] ]
						ifFalse: [ #() ] ] ];
		column: 'Key' 
			evaluated: [:each | GTObjectPrinter new asTruncatedTextFrom: each key ];
		column: 'Value' 
			evaluated: [:each | GTObjectPrinter new asTruncatedTextFrom: each value ];
		showOnly: 50! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:28'!
imageHash
	^ self at: #imageHash! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:28'!
imageHash: anObject
	self at: #imageHash put: anObject! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:29'!
imageVersion
	^ self at: #imageVersion! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:29'!
imageVersion: anObject
	self at: #imageVersion put: anObject! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:29'!
latestUpdate
	^ self at: #latestUpdate! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:30'!
latestUpdate: anObject
	self at: #latestUpdate put: anObject! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 2/28/2015 15:44'!
materializedData
	^ self data ifNotNil: [ :aData | GTSpotterEventRecorder materialize: aData ]! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:30'!
sessionCreationTime
	^ self at: #sessionCreationTime! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:31'!
sessionCreationTime: anObject
	self at: #sessionCreationTime put: anObject! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:30'!
timestamp
	^ self at: #timestamp! !

!GTSpotterRecorderDataAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 3/16/2015 16:30'!
timestamp: anObject
	self at: #timestamp put: anObject! !

Object subclass: #GTSpotterEventRecorder
	instanceVariableNames: 'recordedData mutex process deliveryDelay deliveryQueue shouldDeliver postponedData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Base'!

GTSpotterEventRecorder class
	instanceVariableNames: 'instance'!

!GTSpotterEventRecorder commentStamp: 'AndreiChis 3/2/2015 15:02' prior: 0!
A GTSpotterEventRecorder provides the main functionality for recording and sending usage data in Spotter. The collection and delivery works as follows:

- the recorder registers with Spotter every time Spotter is opened ;
- as long as the setting #sendUsageData is set to false all events received by the recorder are discarded;
- the first time an event is send to the recorder and the setting#sendUsageData is set to true a delivery process is started
- as long as the setting #sendUsageData is set to true all interesting events received by the recorder are stored
- at certain intervals the delivery process collects all data, compressed it, and places a payload to a delivery queue.
- the delivery process takes the next paylod from the queue and attepts to send it to a server; on sucsess the payload is removed from the queue.
- if the setting #sendUsageData is set to false, events are no longer collected and all recorded data is removed next time the delivery process resumes.

The delivery process is not stopped by default after creation. The class size method #cleanUp stops the delivery process and removes all data that still awaits delivery.

Instance Variables
	deliveryDelay:		<Object>
	deliveryQueue:		<Object>
	mutex:		<Object>
	process:		<Object>
	recordedData:		<Object>
	shouldDeliver:		<Object>

deliveryDelay
	- xxxxx

deliveryQueue
	- xxxxx

mutex
	- xxxxx

process
	- xxxxx

recordedData
	- xxxxx

shouldDeliver
	- xxxxx
!

GTSpotterEventRecorder class
	instanceVariableNames: 'instance'!

!GTSpotterEventRecorder class methodsFor: 'accessing' stamp: 'JurajKubelka 2/28/2015 00:45'!
cleanUp
	"Stop and remove the process for sending usage data. Delete all recorder that from the queue."
	
	instance ifNotNil: [ 
		instance stopDelivering ifNotNil: [ :aProcess | 
			[ aProcess terminate ] on: Error do: [ "ignore error" ] ].
		instance := nil ]
! !

!GTSpotterEventRecorder class methodsFor: 'accessing' stamp: 'JurajKubelka 3/6/2015 15:04'!
instance
	"self instance"
	^ instance ifNil: [ instance := self new ]! !

!GTSpotterEventRecorder class methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 15:43'!
materialize: aByteArray
	| materializer stream gzstream materialization |
	stream := aByteArray readStream. 
	gzstream := GZipReadStream on: stream.
	materializer := FLMaterializer newDefault.
	materialization := materializer materializeFrom: gzstream.
	gzstream close.
	stream close.
	^ materialization root.! !

!GTSpotterEventRecorder class methodsFor: 'converting' stamp: 'JurajKubelka 3/28/2015 16:49'!
materializeObjectsFromDirectory: aFileReference
	"self materializeObjectsFromDirectory: './gt/events/' asFileReference"
	^ aFileReference asFileReference allFiles collect: [ :eachFile |
		self materializeObjectsFromFile: eachFile ]  ! !

!GTSpotterEventRecorder class methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 17:37'!
materializeObjectsFromFile: aFileReference 
	"self materializeObjectsFromFile: './gt/events/' asFileReference allFiles any"
	^ self materialize: (aFileReference binaryReadStreamDo: #contents)! !

!GTSpotterEventRecorder class methodsFor: 'accessing' stamp: 'AndreiChis 2/20/2015 10:43'!
reset
	instance := nil! !

!GTSpotterEventRecorder class methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 17:43'!
serialize: anObject
	| serializer stream gzstream |
	stream := ByteArray new writeStream binary. 
	gzstream := GZipWriteStream on: stream.
	serializer := FLSerializer newDefault.
	serializer at: #recorderVersion putAdditionalObject: self version.
	serializer stream: gzstream.
	serializer serialize: anObject.
	gzstream close.
	stream close.
	^ stream contents.! !

!GTSpotterEventRecorder class methodsFor: 'accessing' stamp: 'JurajKubelka 4/10/2015 09:50'!
version
	^ 2
	
"Changes to version 2:
	- GTSpotterRecorderEvent>>timestamp keeps DateAndTime object. 
	  Before, it was an integer as unix time.
"! !

!GTSpotterEventRecorder methodsFor: 'converting' stamp: 'MarcusDenker 7/1/2015 11:53'!
announcementFrom: oldRecordedData
	^ GTSpotterRecorderDataAnnouncement new
			data: (self serialize: oldRecordedData);
			timestamp: DateAndTime now;
			imageHash: SmalltalkImage current imagePath hash;
			sessionCreationTime: SmalltalkImage current session creationTime;
			imageVersion: SystemVersion current version;
			latestUpdate: SystemVersion current highestUpdate;
			computerUUID: GTSpotterEventRecorderSettings uniqueInstance computerUUID asString;
			eventRecorderVersion: GTSpotterEventRecorder version;
			yourself! !

!GTSpotterEventRecorder methodsFor: 'events' stamp: 'JurajKubelka 3/30/2015 14:23'!
buildAndStoreEventFrom: anAnnouncement withType: aClass
	self storeEvent: (aClass fromAnnouncement: anAnnouncement)

	
 	
		
! !

!GTSpotterEventRecorder methodsFor: 'testing' stamp: 'JurajKubelka 3/10/2015 07:22'!
canSendUsageData
	^ GTSpotterEventRecorderSettings sendUsageData! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/26/2015 14:36'!
deliverAndSignalProgress: aBoolean
	self 
		prepareDeliveryData;
		deliverDataAndSignalProgress: aBoolean.
! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/26/2015 17:42'!
deliverData: aByteArray signalProgress: aBoolean
	^ (ZnClient new
			beOneShot;
			timeout: 5;
			signalProgress: aBoolean;
			numberOfRetries: 0;
			url: self remoteURL;
			enforceHttpSuccess: true;
			ifFail: [ :exception | deliveryQueue nextPut: aByteArray ];
			entity: (ZnByteArrayEntity bytes: aByteArray);
			put;
			response)! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/26/2015 14:37'!
deliverDataAndSignalProgress: aBoolean
	deliveryQueue flush: [ :eachByteArray |
		self deliverData: eachByteArray signalProgress: aBoolean ]! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/28/2015 17:45'!
deliverNow
	"for testing purpose"
	deliveryDelay ifNotNil: [ :aDelay | aDelay delaySemaphore signal. aDelay unschedule. ]! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 3/29/2015 14:08'!
deliveryQueue
	"for testing purpose only"
	^ deliveryQueue! !

!GTSpotterEventRecorder methodsFor: 'accessing' stamp: 'AndreiChis 2/24/2015 22:01'!
eventClassNameFor: aSpotterAnnouncementClass

	^  'GTSpotterRecorder', 
		(aSpotterAnnouncementClass name 
			copyFrom: 'GTSpotter' size + 1
			to: aSpotterAnnouncementClass name size), 
		'Event'.
 	
		
! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 3/9/2015 11:31'!
flushRecordedData
	"For debugging purpose only. Remember that those data will not be published."
	| collection |
	collection := OrderedCollection new.
	recordedData flush: [ :eachItem | collection add: eachItem ].
	^ collection! !

!GTSpotterEventRecorder methodsFor: 'initialization' stamp: 'AndreiChis 2/27/2015 23:09'!
initialize
	super initialize.
	
	recordedData := WaitfreeQueue new.
	deliveryQueue := WaitfreeQueue new.
	mutex := Mutex new.! !

!GTSpotterEventRecorder methodsFor: 'initialization' stamp: 'JurajKubelka 3/18/2015 11:35'!
initializeDeliveryProcess
	process := [ [ self shouldDeliver ] whileTrue: [ 
						deliveryDelay := self standardDuration asDelay.
						deliveryDelay wait.
						deliveryDelay := nil.
						self shouldDeliver ifTrue: [ self deliverAndSignalProgress: false ] ] ] 
		forkAt: Processor userBackgroundPriority 
		named: self className, ' regular delivery process'! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/26/2015 14:37'!
lastDataDelivery
	UIManager default informUserDuring: [ :bar |
	  	[ self deliverAndSignalProgress: true ]
	    on: HTTPProgress 
	    do: [ :progress |  
			progress isEmpty ifFalse: [ bar current: progress percentage ].
	      progress resume ] ]
! !

!GTSpotterEventRecorder methodsFor: 'initialization' stamp: 'JurajKubelka 9/1/2015 18:21'!
linkToSpotter: aSpotterModel

	aSpotterModel announcer weak when: Announcement send: #recordAnnouncement: to: self! !

!GTSpotterEventRecorder methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 15:43'!
materialize: aByteArray
	^ self class materialize: aByteArray! !

!GTSpotterEventRecorder methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 15:42'!
materializedAnnouncementFrom: aByteArray 
	^ self materialize: aByteArray! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 3/29/2015 13:39'!
newCollectionWithPostponedData
	^ postponedData 
			ifNil: [ OrderedCollection new ]
			ifNotNil: [ :aCollection | postponedData := nil. aCollection ]
	! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 3/29/2015 18:43'!
postponeSomeData: aCollection 
	| lastExitIndex |
	lastExitIndex := aCollection findLast: [ :each | each isCloseSpotter ].
	lastExitIndex = aCollection size ifTrue: [ ^ aCollection ].
	postponedData := aCollection copyFrom: lastExitIndex + 1 to: aCollection size.
	^ aCollection first: lastExitIndex.! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 3/29/2015 13:30'!
postponedData
	"For testing/debugging purpose."
	^ postponedData! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 3/29/2015 13:59'!
prepareDeliveryData
	self canSendUsageData ifFalse: [ 
		recordedData removeAll.
		deliveryQueue removeAll.
		postponedData := nil.
		^ self ].
	recordedData isEmpty ifTrue: [ ^ self ].
	GTSpotterEventRecorderSettings ensureComputerID.
	self primitivePrepareDeliveryData! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 3/29/2015 13:58'!
primitivePrepareDeliveryData
	| likelyDeliveredData toBeDeliveredData toBeDeliveredAnnouncement |
	likelyDeliveredData := self newCollectionWithPostponedData.
	recordedData flush: [ :eachRecord | likelyDeliveredData addLast: eachRecord ].
	toBeDeliveredData := self postponeSomeData: likelyDeliveredData.
	toBeDeliveredAnnouncement := self serializedAnnouncementFrom: toBeDeliveredData.
	deliveryQueue nextPut: toBeDeliveredAnnouncement.! !

!GTSpotterEventRecorder methodsFor: 'events' stamp: 'JurajKubelka 9/1/2015 18:24'!
recordAccouncement: anAnnouncement
	"Should be deleted in the next iteration."
	^ self recordAnnouncement: anAnnouncement! !

!GTSpotterEventRecorder methodsFor: 'events' stamp: 'JurajKubelka 9/1/2015 18:21'!
recordAnnouncement: anAnnouncement

	self canSendUsageData ifFalse: [ ^ self ].

	"recorderEventClassName := self eventClassNameFor: anAnnouncement class.
	(Smalltalk hasClassNamed: recorderEventClassName) ifFalse: [ ^ false ].
	self storeEvent: (recorderEventClassName asClass fromAnnouncement: anAnnouncement)"
	
	(anAnnouncement respondsTo: #asRecorderEvent) ifTrue: [	
		self storeEvent: anAnnouncement asRecorderEvent ]
	

	
 	
		
! !

!GTSpotterEventRecorder methodsFor: 'events' stamp: 'JurajKubelka 2/26/2015 20:06'!
recordSpotterOpenEvent

	self canSendUsageData ifFalse: [ ^ self ].

	self storeEvent: GTSpotterRecorderOpenEvent new! !

!GTSpotterEventRecorder methodsFor: 'events' stamp: 'JurajKubelka 3/29/2015 14:01'!
recordedData
	"For testing purpose only."
	^ recordedData! !

!GTSpotterEventRecorder methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 13:17'!
remoteURL
	^ GTSpotterEventRecorderSettings url! !

!GTSpotterEventRecorder methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 15:45'!
serialize: anObject
	^ self class serialize: anObject! !

!GTSpotterEventRecorder methodsFor: 'converting' stamp: 'JurajKubelka 2/26/2015 14:16'!
serializedAnnouncementFrom: oldRecordedData
	^ self serialize: (self announcementFrom: oldRecordedData)! !

!GTSpotterEventRecorder methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 12:18'!
shouldDeliver
	^ shouldDeliver ifNil: [ shouldDeliver := true ]! !

!GTSpotterEventRecorder methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 18:32'!
standardDuration
	^ 20 minutes! !

!GTSpotterEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 2/27/2015 19:42'!
stopDelivering
	^ mutex critical: [ | oldProcess |
		shouldDeliver := false.
		deliveryDelay ifNotNil: [ :aDelay | aDelay delaySemaphore signal ].
		oldProcess := process.
		process := nil.
		oldProcess ]! !

!GTSpotterEventRecorder methodsFor: 'events' stamp: 'AndreiChis 2/27/2015 23:15'!
storeEvent: anEvent
	recordedData nextPut: anEvent.
	mutex critical: [ 
		(process isNil or: [ process isTerminated ]) ifTrue: [ self initializeDeliveryProcess ] ]! !

Object subclass: #GTSpotterEventRecorderSettings
	instanceVariableNames: 'computerUUID secretUUID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Settings'!

GTSpotterEventRecorderSettings class
	instanceVariableNames: 'uniqueInstance url sendUsageData'!

!GTSpotterEventRecorderSettings commentStamp: 'AndreiChis 3/2/2015 15:23' prior: 0!
A GTSpotterEventRecorderSettings manages  user preferences related to  sending usage data. This works as follows:  

1. First opening of an image with the recorder
- the first time an image containing this class is opened (or  this class is loaded in an image) a settings file is created in the settings folder of the current user (FileLocator preferences asFileReference).
- a UUID is generated and store d in the setting file to identfy the current computer

2. Opening an image after a settings file was created
- if this image does not already have a setting stored in an image or has a setting with a different UUID then the one of the local computer, any setting present in the image is removed.
- as a second step, if there is a default setting present in the settings file that setting is loaded and stored in the image.

3. Changing the default settings
- the default setting can only be changed from the Agreement UI accessible from the Spotter settings.

This mechanism has the following implications:
- as long as there is no default setting explicitly set a setting has to be set in every image manually
- if there is a default setting stored, whenever opening a new image without an explicit setting the default setting  is saved in the image.

Hence, changing the default setting later on will not change the existing setting in any existing images. It will only apply to new images.
Also copying an image from one computer to another resets it's settings to the default settings of the destination computer.

Instance Variables
	computerUUID:		<Object>
	sendSearchQuery:		<Object>
	sendUsageData:		<Object>
	url:		<Object>
	userAnsweredAgreement:		<Object>

computerUUID
	- xxxxx

sendSearchQuery
	- xxxxx

sendUsageData
	- xxxxx

url
	- xxxxx

userAnsweredAgreement
	- xxxxx
!

GTSpotterEventRecorderSettings class
	instanceVariableNames: 'uniqueInstance url sendUsageData'!

!GTSpotterEventRecorderSettings class methodsFor: 'cleanup' stamp: 'AndreiChis 3/17/2015 17:55'!
cleanUp
	"Clean announcer, unique instance, and settings."
	
	self reset.
	sendUsageData := nil.! !

!GTSpotterEventRecorderSettings class methodsFor: 'load and saving' stamp: 'JurajKubelka 2/26/2015 18:58'!
deletePreferences
	"self deletePreferences"
	[ self preferences ensureDelete ]
	on: Error do: [ "ignore" ]! !

!GTSpotterEventRecorderSettings class methodsFor: 'load and saving' stamp: 'AndreiChis 3/18/2015 14:00'!
ensureComputerID

	self loadPreferences
	! !

!GTSpotterEventRecorderSettings class methodsFor: 'load and saving' stamp: 'JurajKubelka 3/9/2015 11:25'!
loadPreferences
	"self loadPreferences"
	| dictionary newInstance |
	self preferences exists ifFalse: [ 
		"This is a new computer, so we define new computer UUID.
		User still has to agree about sending data if it is not has been done yet."
		^ self savePreferences ].
	[ 
		dictionary := FLMaterializer materializeFromFileNamed: self preferences.
		newInstance := self new dictionary: dictionary.
		"If UUIDs are different, it is likely different user and we should use
		localy stored settings. If the UUIDs are same, we keep settings of the image."
		newInstance computerUUID = self uniqueInstance computerUUID
			ifFalse: [ uniqueInstance := newInstance ]
			ifTrue: [ self uniqueInstance secretUUID: newInstance secretUUID ].
		self saveMissingAttributes: self uniqueInstance dictionary comparingWithOld: dictionary.
	] on: Error do: [ 
		"Preferences likely contains a different settings version, so we store the actual one.
		We should keep the preferences as stable as possible."
		self savePreferences ]! !

!GTSpotterEventRecorderSettings class methodsFor: 'load and saving' stamp: 'JurajKubelka 3/10/2015 10:21'!
persistSettings
	SettingBrowser new exportSettings! !

!GTSpotterEventRecorderSettings class methodsFor: 'origins' stamp: 'JurajKubelka 2/26/2015 18:40'!
preferences
	^ FileLocator preferences / 'org.pharo.gt.spotter.event.recorder.fuel' ! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 2/25/2015 17:04'!
reset
	"self reset"
	uniqueInstance := nil! !

!GTSpotterEventRecorderSettings class methodsFor: 'load and saving' stamp: 'JurajKubelka 3/9/2015 11:19'!
saveMissingAttributes: imageDictionary comparingWithOld: diskDictionary
	"It saves to the disk items that are included in imageDictionary,
	but are missing in diskDictionary. Useful when a new version of the event
	recorder adds new configuration values that should be saved too."
	| difference newDiskDictionary |
	difference := imageDictionary keys difference: diskDictionary keys.
	difference ifEmpty: [ ^ self ].
	newDiskDictionary := diskDictionary copy.
	difference do: [ :eachKey |
		newDiskDictionary at: eachKey put: (imageDictionary at: eachKey) ].
	self savePreferences: newDiskDictionary.! !

!GTSpotterEventRecorderSettings class methodsFor: 'load and saving' stamp: 'JurajKubelka 3/9/2015 11:05'!
savePreferences
	"self savePreferences"
	self savePreferences: self uniqueInstance dictionary.! !

!GTSpotterEventRecorderSettings class methodsFor: 'load and saving' stamp: 'JurajKubelka 3/9/2015 11:05'!
savePreferences: aDictionary
	"self savePreferences: Dictionary new."
	self deletePreferences.
	[ FLSerializer serialize: aDictionary toFileNamed: self preferences ]
		on: Error do: [ "ignore" ]! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 3/10/2015 07:16'!
sendUsageData
	^ sendUsageData ifNil: [ sendUsageData := false ]! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'AndreiChis 3/18/2015 13:58'!
sendUsageData: aBoolean
	sendUsageData := aBoolean.! !

!GTSpotterEventRecorderSettings class methodsFor: 'settings' stamp: 'MarcusDenker 9/16/2015 10:27'!
settingsOnSendUsageData: aBuilder 
	<systemsettings>
	
	(aBuilder setting: #sendUsageData)
		parent: #gt;
		target: self;
		description: 'If checked, anonymous usage data is collected and regularly send to a remote server. Only information about how you use the GTools is collected (e.g., open, close, select, enter search query, dive in result). No method names, class names or any other piece of code is collected.' translated;
		label: 'Send anonymous usage data'
! !

!GTSpotterEventRecorderSettings class methodsFor: 'settings' stamp: 'JurajKubelka 2/25/2015 14:38'!
settingsOnUrl: aBuilder 
	<systemsettings>
	
	(aBuilder setting: #stringUrl)
		parent: #gt;
		target: self;
		description: 'The url for the Spotter Event Recorder server.' translated;
		label: 'Spotter Event Recorder URL address'
! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 2/27/2015 16:08'!
stringUrl
	^ self url asString! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 2/27/2015 16:08'!
stringUrl: aUrlString
	^ self url: aUrlString! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 3/29/2015 14:33'!
uniqueInstance
	"self uniqueInstance"
	^ uniqueInstance ifNil: [ 
			uniqueInstance := self new ].! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 2/27/2015 16:08'!
url
	^ url ifNil: [ url := 'http://gc.dcc.uchile.cl.:8080/gt/events' asZnUrl ]! !

!GTSpotterEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 3/10/2015 07:16'!
url: aZnUrl
	url := aZnUrl ifNotNil: [ aZnUrl asZnUrl ].! !

!GTSpotterEventRecorderSettings methodsFor: 'accessing' stamp: 'JurajKubelka 2/26/2015 18:39'!
computerUUID
	^ computerUUID! !

!GTSpotterEventRecorderSettings methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 15:55'!
dictionary
	| dictionary |
	dictionary := Dictionary new: self class instSize.
	1 to: self class instSize do: [ :index |
		dictionary 
			at: (self class instVarNameForIndex: index)
			put: (self instVarAt: index)
	].
	^ dictionary! !

!GTSpotterEventRecorderSettings methodsFor: 'converting' stamp: 'JurajKubelka 2/28/2015 16:04'!
dictionary: aCollection 
	aCollection keysAndValuesDo: [ :key :value | | instVarIndex |
		instVarIndex := self class instVarIndexFor: key.
		instVarIndex isZero ifFalse: [ 
			self instVarAt: instVarIndex put: value
		]
	]! !

!GTSpotterEventRecorderSettings methodsFor: 'initialization' stamp: 'JurajKubelka 3/6/2015 15:00'!
initialize
	super initialize.
	computerUUID := UUID new.
	secretUUID := UUID new.! !

!GTSpotterEventRecorderSettings methodsFor: 'accessing' stamp: 'JurajKubelka 3/6/2015 15:00'!
secretUUID
	^ secretUUID! !

!GTSpotterEventRecorderSettings methodsFor: 'accessing' stamp: 'JurajKubelka 3/9/2015 11:25'!
secretUUID: anUUID
	secretUUID := anUUID! !

Object subclass: #GTSpotterRecorderEvent
	instanceVariableNames: 'timestamp info'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderEvent commentStamp: 'AndreiChis 3/2/2015 15:30' prior: 0!
GTSpotterRecorderEvent is the root class for representing usage data generated by Spotter. 
All events have a timestamp.!

GTSpotterRecorderEvent subclass: #GTSpotterRecorderActOnEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderActOnEvent commentStamp: 'AndreiChis 2/27/2015 14:35' prior: 0!
A GTSpotterRecorderActOnEvent models the action of pressing enter (or double clicking) on an element.
!

!GTSpotterRecorderActOnEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:52'!
accept: aVisitor
	^ aVisitor visitActOnEvent: self! !

!GTSpotterRecorderActOnEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/31/2015 10:48'!
closeActionSymbol
	^ #actOn! !

!GTSpotterRecorderActOnEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:55'!
elementName
	^ self infoAt: #elementName! !

!GTSpotterRecorderActOnEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:55'!
elementType
	^ self infoAt: #elementType! !

!GTSpotterRecorderActOnEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:55'!
eventType
	^ self infoAt: #eventType! !

!GTSpotterRecorderActOnEvent methodsFor: 'initialization' stamp: 'JurajKubelka 3/23/2015 12:23'!
fromAnnouncement: anAnnouncement
	super fromAnnouncement: anAnnouncement.
	self infoAt: #eventType put: anAnnouncement eventType.
	self infoAt: #elementName putHashForText: anAnnouncement element asString.
	self infoAt: #elementType putHashForText: anAnnouncement element class name asString.! !

!GTSpotterRecorderActOnEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 18:43'!
isCloseSpotter
	^ true! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderCategorySelectedEvent
	instanceVariableNames: 'categoryLabel elementCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderCategorySelectedEvent commentStamp: 'AndreiChis 2/27/2015 14:41' prior: 0!
A GTSpotterRecorderCategorySelectedEvent  models the action of  selecting a category in the search results. This event stores the name of the category and the number of results that were matched by the current query.

Instance Variables
	categoryLabel:		<Object>
	elementCount:		<Object>

categoryLabel
	- xxxxx

elementCount
	- xxxxx
!

!GTSpotterRecorderCategorySelectedEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:52'!
accept: aVisitor
	^ aVisitor visitCategorySelectedEvent: self! !

!GTSpotterRecorderCategorySelectedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:35'!
categoryLabel
	^ categoryLabel! !

!GTSpotterRecorderCategorySelectedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:35'!
elementCount
	^ elementCount! !

!GTSpotterRecorderCategorySelectedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/30/2015 17:08'!
elementCountOrZero
	^ self elementCount ifNil: [ 0 ]! !

!GTSpotterRecorderCategorySelectedEvent methodsFor: 'initialization' stamp: 'AndreiChis 2/24/2015 21:36'!
fromAnnouncement: anAnnouncement

	categoryLabel := anAnnouncement category title.
	elementCount := anAnnouncement category allFilteredCandidates size.! !

!GTSpotterRecorderCategorySelectedEvent methodsFor: 'gt-inspector-extension' stamp: 'JurajKubelka 3/28/2015 23:37'!
gtInspectorItems
	^ OrderedCollection new
			add: (#categoryLabel -> self categoryLabel);
			add: (#elementCount -> self elementCount);
			addAll: super gtInspectorItems;
			yourself! !

!GTSpotterRecorderCategorySelectedEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/30/2015 17:41'!
isCategorySelection
	^ true! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderContextChangedEvent
	instanceVariableNames: 'isRefinement currentTextInfo previousTextInfo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderContextChangedEvent commentStamp: 'AndreiChis 3/2/2015 15:29' prior: 0!
A GTSpotterRecorderContextChangedEvent stores current query written in Spotter input field. This event stores the size of the query and wheather or not the current query is a refinment of a previous query.

Instance Variables
	text:		<String>

text
	- query writen in Spotter input field
!

!GTSpotterRecorderContextChangedEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:52'!
accept: aVisitor
	^ aVisitor visitContextChangedEvent: self! !

!GTSpotterRecorderContextChangedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/30/2015 12:23'!
currentTextHashes
	^ self stringHashesFor: self currentTextInfo! !

!GTSpotterRecorderContextChangedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:38'!
currentTextInfo
	^ currentTextInfo! !

!GTSpotterRecorderContextChangedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 4/10/2015 14:49'!
currentTextSizes
	^ self stringSizesFor: self currentTextInfo! !

!GTSpotterRecorderContextChangedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 4/11/2015 23:46'!
currentTextSpecials
	^ self stringSpecialsFor: self currentTextInfo! !

!GTSpotterRecorderContextChangedEvent methodsFor: 'initialization' stamp: 'JurajKubelka 3/6/2015 15:28'!
fromAnnouncement: anAnnouncement

	currentTextInfo := self textInfoFrom: anAnnouncement currentText.
	anAnnouncement previousText size == 0 ifTrue: [ ^ self ].
	previousTextInfo := self textInfoFrom: anAnnouncement previousText.
	isRefinement := (anAnnouncement currentText size > anAnnouncement previousText size 
		ifTrue: [ anAnnouncement currentText beginsWith: anAnnouncement previousText]
		ifFalse: [ anAnnouncement previousText beginsWith: anAnnouncement currentText ]).! !

!GTSpotterRecorderContextChangedEvent methodsFor: 'gt-inspector-extension' stamp: 'JurajKubelka 3/28/2015 23:38'!
gtInspectorItems
	^ OrderedCollection new
			add: (#isRefinement -> self isRefinement);
			add: (#currentTextInfo -> self currentTextInfo);
			add: (#previousTextInfo -> self previousTextInfo);
			addAll: super gtInspectorItems;
			yourself! !

!GTSpotterRecorderContextChangedEvent methodsFor: 'testing' stamp: 'JurajKubelka 4/11/2015 23:48'!
hasCurrentTextSpecials
	^ self currentTextSpecials includes: true! !

!GTSpotterRecorderContextChangedEvent methodsFor: 'initialization' stamp: 'JurajKubelka 2/28/2015 11:59'!
initialize
	super initialize.
	isRefinement := false.! !

!GTSpotterRecorderContextChangedEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/30/2015 11:44'!
isContextChanged
	^ self currentTextInfo notNil! !

!GTSpotterRecorderContextChangedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:38'!
isRefinement
	^ isRefinement! !

!GTSpotterRecorderContextChangedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:38'!
previousTextInfo
	^ previousTextInfo! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderDiveInCategoryEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderDiveInCategoryEvent commentStamp: 'AndreiChis 2/27/2015 14:36' prior: 0!
A GTSpotterRecorderDiveInCategoryEvent  models the action of  diving in a category (list of results).
!

!GTSpotterRecorderDiveInCategoryEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:53'!
accept: aVisitor
	^ aVisitor visitDiveInCategoryEvent: self! !

!GTSpotterRecorderDiveInCategoryEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:54'!
allFilteredCandidates
	^ self infoAt: #allFilteredCandidates! !

!GTSpotterRecorderDiveInCategoryEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:54'!
categoryTitle
	^ self infoAt: #categoryTitle! !

!GTSpotterRecorderDiveInCategoryEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:54'!
eventType
	^ self infoAt: #eventType! !

!GTSpotterRecorderDiveInCategoryEvent methodsFor: 'initialization' stamp: 'JurajKubelka 3/23/2015 11:52'!
fromAnnouncement: anAnnouncement
	super fromAnnouncement: anAnnouncement.
	self infoAt: #eventType put: anAnnouncement eventType.
	self infoAt: #categoryTitle put: anAnnouncement category title.
	self infoAt: #allFilteredCandidates put: anAnnouncement category allFilteredCandidates size.! !

!GTSpotterRecorderDiveInCategoryEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 19:14'!
isDiveIn
	^ true! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderDiveInElementEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderDiveInElementEvent commentStamp: 'AndreiChis 2/27/2015 14:36' prior: 0!
A GTSpotterRecorderDiveInElementEvent models the action of diving in a selected element.
!

!GTSpotterRecorderDiveInElementEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:53'!
accept: aVisitor
	^ aVisitor visitDiveInElementEvent: self! !

!GTSpotterRecorderDiveInElementEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:53'!
elementName
	^ self infoAt: #elementName! !

!GTSpotterRecorderDiveInElementEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:53'!
elementType
	^ self infoAt: #elementType! !

!GTSpotterRecorderDiveInElementEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:53'!
eventType
	^ self infoAt: #eventType! !

!GTSpotterRecorderDiveInElementEvent methodsFor: 'initialization' stamp: 'JurajKubelka 3/23/2015 12:25'!
fromAnnouncement: anAnnouncement
	super fromAnnouncement: anAnnouncement.
	self infoAt: #eventType put: anAnnouncement eventType.
	self infoAt: #elementName putHashForText: anAnnouncement element asString.
	self infoAt: #elementType putHashForText: anAnnouncement element class name asString.
! !

!GTSpotterRecorderDiveInElementEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 19:15'!
isDiveIn
	^ true! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderDiveOutEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderDiveOutEvent commentStamp: 'AndreiChis 2/27/2015 14:37' prior: 0!
A GTSpotterRecorderDiveOutEvent  models the action of  diving out a step.
!

!GTSpotterRecorderDiveOutEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:53'!
accept: aVisitor
	^ aVisitor visitDiveOutEvent: self! !

!GTSpotterRecorderDiveOutEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:55'!
elementName
	^ self infoAt: #elementName! !

!GTSpotterRecorderDiveOutEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:55'!
elementType
	^ self infoAt: #elementType! !

!GTSpotterRecorderDiveOutEvent methodsFor: 'initialization' stamp: 'JurajKubelka 3/23/2015 12:26'!
fromAnnouncement: anAnnouncement
	super fromAnnouncement: anAnnouncement.
	self infoAt: #elementName putHashForText: anAnnouncement element asString.
	self infoAt: #elementType putHashForText: anAnnouncement element class name asString.! !

!GTSpotterRecorderDiveOutEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 19:15'!
isDiveOut
	^ true! !

!GTSpotterRecorderEvent class methodsFor: 'instance creation' stamp: 'AndreiChis 2/24/2015 21:08'!
fromAnnouncement: anAnnouncement

	^ self new fromAnnouncement: anAnnouncement! !

!GTSpotterRecorderEvent class methodsFor: 'instance creation' stamp: 'StefanReichhart 4/9/2015 12:01'!
gtExampleAnySubclass
	<gtExample>
	^ self subclasses anyOne new ! !

!GTSpotterRecorderEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:49'!
accept: aVisitor
	^ self subclassResponsibility! !

!GTSpotterRecorderEvent methodsFor: 'accessing - announcement info' stamp: 'JurajKubelka 3/29/2015 17:32'!
announcementTimestamp
	^ self infoAt: #timestamp! !

!GTSpotterRecorderEvent methodsFor: 'converting' stamp: 'JurajKubelka 3/28/2015 23:13'!
associations
	^ info 
			ifNil: [ #() ]
			ifNotNil: [ info associations ]! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/31/2015 10:49'!
closeActionSymbol
	^ #improper! !

!GTSpotterRecorderEvent methodsFor: 'accessing - announcement info' stamp: 'JurajKubelka 3/29/2015 17:33'!
computerUUID
	^ self infoAt: #computerUUID! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 4/10/2015 10:38'!
dateAndTime
	| aTimestamp |
	self infoAt: #dateAndTime ifPresent: [ :obj | ^ obj ] ifAbsent: [ "ignore" ].
	aTimestamp := self timestamp.
	aTimestamp ifNil: [ ^ aTimestamp ].
	
	aTimestamp isNumber 
		ifTrue: [ "version 1 keeps unix time"
			"We do not have information about time zone."
			^ self 
				infoAt: #dateAndTime 
				put: ((DateAndTime fromUnixTime: aTimestamp) translateTo: 0) ].
	
	"since version 2, timestamp keeps DateAndTime object."
	^ aTimestamp! !

!GTSpotterRecorderEvent methodsFor: 'accessing - announcement info' stamp: 'JurajKubelka 3/29/2015 17:33'!
eventRecorderVersion
	^ self infoAt: #eventRecorderVersion! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/29/2015 17:32'!
eventTimestamp
	^ self timestamp! !

!GTSpotterRecorderEvent methodsFor: 'initialization' stamp: 'AndreiChis 2/24/2015 21:08'!
fromAnnouncement: anAnnouncement

	! !

!GTSpotterRecorderEvent methodsFor: 'gt-inspector-extension' stamp: 'JurajKubelka 3/28/2015 23:34'!
gtInspectorItems
	^ OrderedCollection
		 with: (#eventTimestamp -> self timestamp) withAll: self associations! !

!GTSpotterRecorderEvent methodsFor: 'gt-inspector-extension' stamp: 'JurajKubelka 3/30/2015 11:47'!
gtInspectorItemsIn: composite
	<gtInspectorPresentationOrder: -1>  
	^ (composite table)
		title: 'Info';
		display: [ :anEvent | anEvent gtInspectorItems ];
		children: [ :each | 
			(each value isDictionary) 
				ifTrue: [ each value associations ] 
				ifFalse: [ (each value isArray and: [ each value allSatisfy: #isDictionary ])
						ifTrue: [ each value collectWithIndex: [:x :i | i -> x ] ]
						ifFalse: [ #() ] ] ];
		column: 'Key' 
			evaluated: [:each | GTObjectPrinter new asTruncatedTextFrom: each key ];
		column: 'Value' 
			evaluated: [:each | GTObjectPrinter new asTruncatedTextFrom: each value ];
		beMultiple;
		send: [ :selection | 
			(selection notNil and: [ selection size = 1 ]) 
				ifTrue: [ selection anyOne value ] 
				ifFalse: [ selection collect: #value ] ];
		showOnly: 50! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/23/2015 12:14'!
hashForText: aString
	^ aString hash bitXor: self settings secretUUID hash! !

!GTSpotterRecorderEvent methodsFor: 'accessing - announcement info' stamp: 'JurajKubelka 3/29/2015 17:05'!
imageHash
	^ self infoAt: #imageHash! !

!GTSpotterRecorderEvent methodsFor: 'accessing - announcement info' stamp: 'JurajKubelka 3/29/2015 17:33'!
imageVersion
	^ self infoAt: #imageVersion! !

!GTSpotterRecorderEvent methodsFor: 'private' stamp: 'JurajKubelka 4/1/2015 09:48'!
info
	"For the testing purpose only; we use it in #instancesForAllEvents.
	We want to instantiate info only when it is really necessary."
	^ info ifNil: [ info := Dictionary new ].! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/23/2015 11:36'!
infoAt: key
	^ self infoAt: key ifAbsent: [ nil ]! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/23/2015 11:36'!
infoAt: key ifAbsent: absentBlock
	info ifNil: [ ^ absentBlock value].
	^ info at: key ifAbsent: absentBlock! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/31/2015 13:34'!
infoAt: key ifAbsentPut: putBlock
	info ifNil: [ info := Dictionary new ].
	^ info at: key ifAbsentPut: putBlock! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/30/2015 12:18'!
infoAt: key ifPresent: presentBlock ifAbsent: absentBlock
	info ifNil: [ ^ absentBlock value].
	^ info at: key ifPresent: presentBlock ifAbsent: absentBlock! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/23/2015 11:36'!
infoAt: key put: value
	info ifNil: [ info := Dictionary new ].
	^ info at: key put: value! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/23/2015 12:16'!
infoAt: key putHashForText: valueString 
	^ self 
		infoAt: key 
		put: (self hashForText: valueString)! !

!GTSpotterRecorderEvent methodsFor: 'initialization' stamp: 'JurajKubelka 4/10/2015 09:48'!
initialize
	super initialize.
	timestamp := DateAndTime now.! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/30/2015 17:41'!
isCategorySelection
	^ false! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 18:43'!
isCloseSpotter
	^ false! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/30/2015 11:41'!
isContextChanged
	^ false! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/31/2015 12:02'!
isDive
	^ self isDiveIn or: [ self isDiveOut ]! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 19:14'!
isDiveIn
	^ false! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 19:14'!
isDiveOut
	^ false! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 18:44'!
isHidePreview
	^ false! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/30/2015 14:30'!
isOpenOrCloseSpotter
	^ self isOpenSpotter or: [ self isCloseSpotter ]! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 18:43'!
isOpenSpotter
	^ false! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/30/2015 11:51'!
isSearchFinished
	^ false! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 18:44'!
isShowPreview
	^ false! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 19:40'!
isUserSelection
	^ false! !

!GTSpotterRecorderEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/31/2015 11:35'!
isUserSelectionClass
	^ false! !

!GTSpotterRecorderEvent methodsFor: 'accessing - announcement info' stamp: 'JurajKubelka 3/29/2015 17:33'!
latestUpdate
	^ self infoAt: #latestUpdate! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 4/9/2015 11:19'!
removeInfoKey: key
	^ self removeInfoKey: key ifAbsent: [ "ignore" ]! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 4/9/2015 11:18'!
removeInfoKey: key ifAbsent: absentBlock
	info ifNil: [ ^ absentBlock value].
	^ info removeKey: key ifAbsent: absentBlock! !

!GTSpotterRecorderEvent methodsFor: 'accessing - announcement info' stamp: 'JurajKubelka 3/29/2015 17:33'!
sessionCreationTime
	^ self infoAt: #sessionCreationTime! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/6/2015 15:08'!
settings
	^ GTSpotterEventRecorderSettings uniqueInstance! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/30/2015 12:18'!
stringHashesAt: aSymbol
	^ self 
		infoAt: aSymbol
		ifPresent: [ :array | array first ]
		ifAbsent: [ #() ]! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/30/2015 12:22'!
stringHashesFor: anArrayOrNil
	^ anArrayOrNil 
			ifNil: [ #() ]
			ifNotNil: [ :array | array first ]! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/30/2015 12:19'!
stringSizesAt: aSymbol
	^ self 
		infoAt: aSymbol
		ifPresent: [ :array | array second ]
		ifAbsent: [ #() ]! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/30/2015 12:22'!
stringSizesFor: anArrayOrNil
	^ anArrayOrNil 
			ifNil: [ #() ]
			ifNotNil: [ :array | array second ]! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/30/2015 12:19'!
stringSpecialsAt: aSymbol
	^ self 
		infoAt: aSymbol
		ifPresent: [ :array | array third ]
		ifAbsent: [ #() ]! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/30/2015 12:22'!
stringSpecialsFor: anArrayOrNil
	^ anArrayOrNil 
			ifNil: [ #() ]
			ifNotNil: [ :array | array third ]! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/23/2015 12:16'!
textInfoFrom: aString
	| strings hashes sizes specials |
	aString isNil ifTrue: [ ^ nil ].
	aString size isZero ifTrue: [ ^ nil ].
	strings := aString substrings.
	hashes := strings collect: [ :each | self hashForText: each ] as: Array.
	sizes := strings collect: #size as: Array.
	specials := strings collect: [ :each | each beginsWith: '#' ] as: Array.
	^ Array with: hashes with: sizes with: specials ! !

!GTSpotterRecorderEvent methodsFor: 'accessing' stamp: 'JurajKubelka 4/10/2015 09:39'!
timestamp
	^ timestamp! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderExitAnnouncementEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderExitAnnouncementEvent commentStamp: 'AndreiChis 2/27/2015 14:37' prior: 0!
A GTSpotterRecorderExitAnnouncementEvent  models the action of  exiting Spotter. 

Instance Variables
	timestamp:		<Object>

timestamp
	- xxxxx
!

!GTSpotterRecorderExitAnnouncementEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:53'!
accept: aVisitor
	^ aVisitor visitExitEvent: self! !

!GTSpotterRecorderExitAnnouncementEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/31/2015 10:49'!
closeActionSymbol
	^ self eventType! !

!GTSpotterRecorderExitAnnouncementEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/31/2015 12:38'!
eventType
	^ self infoAt: #eventType ifAbsent: [ #missing ]! !

!GTSpotterRecorderExitAnnouncementEvent methodsFor: 'initialization' stamp: 'JurajKubelka 3/23/2015 12:04'!
fromAnnouncement: anAnnouncement
	super fromAnnouncement: anAnnouncement.
	self infoAt: #eventType put: anAnnouncement eventType.

! !

!GTSpotterRecorderExitAnnouncementEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 18:43'!
isCloseSpotter
	^ true! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderHidePreviewEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderHidePreviewEvent commentStamp: 'AndreiChis 2/27/2015 14:38' prior: 0!
A GTSpotterRecorderHidePreviewEvent  models the action of explicitly hidding the preview.
!

!GTSpotterRecorderHidePreviewEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:53'!
accept: aVisitor
	^ aVisitor visitHidePreviewEvent: self! !

!GTSpotterRecorderHidePreviewEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 18:44'!
isHidePreview
	^ true! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderOpenEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderOpenEvent commentStamp: 'AndreiChis 2/27/2015 14:38' prior: 0!
A GTSpotterRecorderOpenEvent  models the action of  opening Spotter.

Instance Variables
	timestamp:		<Object>

timestamp
	- xxxxx
!

!GTSpotterRecorderOpenEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:53'!
accept: aVisitor
	^ aVisitor visitOpenEvent: self! !

!GTSpotterRecorderOpenEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 18:43'!
isOpenSpotter
	^ true! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderSearchFinishedEvent
	instanceVariableNames: 'query processors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderSearchFinishedEvent commentStamp: 'AndreiChis 3/2/2015 15:30' prior: 0!
A GTSpotterRecorderSearchFinishedEvent is triggered when spotter finished a search and displayed all the results. This event stores the size of the query.
!

!GTSpotterRecorderSearchFinishedEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:53'!
accept: aVisitor
	^ aVisitor visitSearchFinishedEvent: self! !

!GTSpotterRecorderSearchFinishedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/30/2015 16:52'!
categories
	^ self processors 
			ifNotNil: [ :aCollection | aCollection select: #isString ]
			ifNil: [ #() ]! !

!GTSpotterRecorderSearchFinishedEvent methodsFor: 'initialization' stamp: 'JurajKubelka 3/6/2015 16:48'!
fromAnnouncement: anAnnouncement
	query := self textInfoFrom: anAnnouncement query.
	processors := (anAnnouncement processors flatCollect: [ :eachProcessor |
		Array 
			with: eachProcessor title
			with: eachProcessor allFilteredCandidates size ]) asArray! !

!GTSpotterRecorderSearchFinishedEvent methodsFor: 'gt-inspector-extension' stamp: 'JurajKubelka 3/28/2015 23:40'!
gtInspectorItems
	^ OrderedCollection new
			add: (#query -> self query);
			add: (#processors -> self processors);
			addAll: super gtInspectorItems;
			yourself! !

!GTSpotterRecorderSearchFinishedEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/30/2015 11:52'!
isSearchFinished
	^ self query notNil! !

!GTSpotterRecorderSearchFinishedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/30/2015 16:53'!
items
	^ self processors 
			ifNotNil: [ :aCollection | aCollection select: #isNumber ]
			ifNil: [ #() ]! !

!GTSpotterRecorderSearchFinishedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:39'!
processors
	^ processors! !

!GTSpotterRecorderSearchFinishedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:39'!
query
	^ query! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderSelectedCandidateChangedEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderSelectedCandidateChangedEvent commentStamp: 'AndreiChis 2/28/2015 19:07' prior: 0!
A GTSpotterRecorderSelectedCandidateChangedEvent models the action of  selecting an element in the search results.
!

!GTSpotterRecorderSelectedCandidateChangedEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:53'!
accept: aVisitor
	^ aVisitor visitSelectedCandidateChangedEvent: self! !

!GTSpotterRecorderSelectedCandidateChangedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:56'!
elementName
	^ self infoAt: #elementName! !

!GTSpotterRecorderSelectedCandidateChangedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:56'!
elementType
	^ self infoAt: #elementType! !

!GTSpotterRecorderSelectedCandidateChangedEvent methodsFor: 'accessing' stamp: 'JurajKubelka 3/28/2015 23:56'!
eventType
	^ self infoAt: #eventType! !

!GTSpotterRecorderSelectedCandidateChangedEvent methodsFor: 'initialization' stamp: 'JurajKubelka 3/23/2015 12:29'!
fromAnnouncement: anAnnouncement
	super fromAnnouncement: anAnnouncement.
	self infoAt: #eventType put: anAnnouncement eventType.
	self infoAt: #elementName putHashForText: anAnnouncement selected asString.
	self infoAt: #elementType putHashForText: anAnnouncement selected class name asString.
! !

!GTSpotterRecorderSelectedCandidateChangedEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 19:42'!
isUserSelection
	^ self eventType notNil and: [ (self eventType = #unknown) not ]! !

!GTSpotterRecorderSelectedCandidateChangedEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/31/2015 11:35'!
isUserSelectionClass
	^ true! !

GTSpotterRecorderEvent subclass: #GTSpotterRecorderShowPreviewEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Spotter-EventRecorder-Events'!

!GTSpotterRecorderShowPreviewEvent commentStamp: 'AndreiChis 2/27/2015 14:39' prior: 0!
A GTSpotterRecorderShowPreviewEvent models the action of explicitly closing the preview of an object.
!

!GTSpotterRecorderShowPreviewEvent methodsFor: 'visitor' stamp: 'JurajKubelka 4/11/2015 16:54'!
accept: aVisitor
	^ aVisitor visitShowPreviewEvent: self! !

!GTSpotterRecorderShowPreviewEvent methodsFor: 'testing' stamp: 'JurajKubelka 3/29/2015 18:44'!
isShowPreview
	^ true! !

!GTSpotterExitAnnouncement methodsFor: '*gt-spotter-eventrecorder' stamp: 'YuriyTymchuk 8/30/2015 01:45'!
asRecorderEvent

	^ GTSpotterRecorderExitAnnouncementEvent fromAnnouncement: self! !
